/*
 * CC3100_board.h - TM4C1294 launchpad configuration
 *
 * Copyright (c) 2015 Texas Instruments Incorporated - http://www.ti.com/
 * ALL RIGHTS RESERVED
 *
 */


#ifndef _CC3100_BOARD_H
#define    _CC3100_BOARD_H


#define PIN_HIGH                              0xFF
#define PIN_LOW                              (!PIN_HIGH)

typedef void (*P_EVENT_HANDLER)(void* pValue);

/*!
    \brief register an interrupt handler for the host IRQ

    \param[in]      InterruptHdl    -    pointer to interrupt handler function

    \param[in]      pValue          -    pointer to a memory strcuture that is
                    passed to the interrupt handler.

    \return         upon successful registration, the function shall return 0.
                    Otherwise, -1 shall be returned

    \sa
    \note           If there is already registered interrupt handler, the
                    function should overwrite the old handler with the new one

    \warning
*/
int registerInterruptHandler(P_EVENT_HANDLER InterruptHdl , void* pValue);

/*!
    \brief          GPIOM interrupt handler

    \param[in]      none

    \return         none

    \note

    \warning
*/
void GPIOM_intHandler();

/*!
    \brief             Enables the CC3100

    \param[in]         none

    \return            none

    \note

    \warning
*/
void CC3100_enable();

/*!
    \brief             Disables the CC3100

    \param[in]         none

    \return            none

    \note

    \warning
*/
void CC3100_disable();

/*!
    \brief          Enables the interrupt from the CC3100

    \param[in]      none

    \return         none

    \note

    \warning
*/
void CC3100_InterruptEnable();

/*!
    \brief          Disables the interrupt from the CC3100

    \param[in]      none

    \return         none

    \note

    \warning
*/
void CC3100_InterruptDisable();

/*!
    \brief          Stops the Watch Dog timer

    \param[in]      none

    \return         none

    \note

    \warning
*/
void stopWDT();

/*!
    \brief          Initialize the system clock of MCU

    \param[in]      none

    \return         none

    \note

    \warning
*/
void initClk();

/*!
    \brief      Masks the Host IRQ

	\param[in]      none

    \return         none

    \warning
*/
void MaskIntHdlr();

/*!
    \brief     Unmasks the Host IRQ

	\param[in]      none

    \return         none

    \warning
*/
void UnMaskIntHdlr();

/*!
    \brief          GPIOB interrupt handler

    \param[in]      none

    \return         none

    \note

    \warning
*/
void UART1_intHandler();

/*!
    \brief     Produce delay in ms

    \param[in]         interval - Time in ms

    \return            none

    \note

    \warning
*/
void Delay(unsigned long interval);

typedef enum
{
    NO_LED,
    LED1,
    LED2
} ledEnum;

void initLEDs();

/*!
    \brief          Turn on the LED on MCU

    \param[in]      ledNum - LED to be turned on

    \return         none

    \note

    \warning
*/
void turnLedOn(char ledNum);

/*!
    \brief          Turn off the LED on MCU

    \param[in]      ledNum - LED to be turned of

    \return         none

    \note

    \warning
*/
void turnLedOff(char ledNum);

/*!
    \brief          Toggle the LED

    \param[in]      ledNum - LED to be toggled

    \return         none

    \note

    \warning
*/
void toggleLed(char ledNum);

/*!
    \brief          Get the LED status

    \param[in]      none

    \return         unsigned char - byte containing the status of LED

    \note

    \warning
*/
unsigned char GetLEDStatus();

/*!
    \brief     Produce delay in ms

    \param[in]         interval - Time in ms

    \return            none

    \note

    \warning
*/

typedef enum
{
    ANT1,
    ANT2
} antenna;

void SelAntenna(int antenna);
void initAntSelGPIO();


#endif
