/*
 * CC3100_spi.c - TM4C1294 launchpad spi interface implementation
 *
 * Copyright (c) 2015 Texas Instruments Incorporated - http://www.ti.com/
 * ALL RIGHTS RESERVED
 *
 */

#ifndef SL_IF_TYPE_UART
#include "simplelink.h"
#include "CC3100_board.h"
#include "CC3100_spi.h"
#include "inc/hw_memmap.h"
#include "inc/hw_ssi.h"
#include "inc/hw_gpio.h"
#include "inc/hw_types.h"
#include "driverlib/pin_map.h"
#include "inc/hw_ints.h"
#include "driverlib/ssi.h"
#include "driverlib/rom.h"
#include "driverlib/gpio.h"
#include "driverlib/sysctl.h"
#include "driverlib/interrupt.h"

#define ASSERT_CS()         GPIOPinWrite(GPIO_PORTP_BASE,GPIO_PIN_5, PIN_LOW)
#define DEASSERT_CS()       GPIOPinWrite(GPIO_PORTP_BASE,GPIO_PIN_5, PIN_HIGH)
extern uint32_t g_ui32SysClock;


int spi_Close(Fd_t fd)
{
    /* Disable WLAN Interrupt ... */
    CC3100_InterruptDisable();

    return NONOS_RET_OK;
}

Fd_t spi_Open(char *ifName, unsigned long flags)
{

    /* Configure CS (PP5) and nHIB (PD4) lines */
    SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOD);
    SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOP);
    GPIOPinTypeGPIOOutput(GPIO_PORTD_BASE, GPIO_PIN_4);
    GPIOPinTypeGPIOOutput(GPIO_PORTP_BASE, GPIO_PIN_5);
    GPIOPinWrite(GPIO_PORTD_BASE,GPIO_PIN_4, PIN_LOW);
    GPIOPinWrite(GPIO_PORTP_BASE,GPIO_PIN_5, PIN_HIGH);

    SysCtlPeripheralEnable(SYSCTL_PERIPH_SSI3);
    SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOM);
    SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOQ);

    GPIOPinConfigure(GPIO_PQ0_SSI3CLK);
    GPIOPinConfigure(GPIO_PQ1_SSI3FSS);
    GPIOPinConfigure(GPIO_PQ2_SSI3XDAT0);
    GPIOPinConfigure(GPIO_PQ3_SSI3XDAT1);

    GPIOPinTypeSSI(GPIO_PORTQ_BASE, GPIO_PIN_0 | GPIO_PIN_1 | GPIO_PIN_2
                   | GPIO_PIN_3);

    /* CLOCK IS CONFIGURED AT 12 MHz */
    SSIConfigSetExpClk(SSI3_BASE, g_ui32SysClock, SSI_FRF_MOTO_MODE_0,
                       SSI_MODE_MASTER, 12000000, 8);

    SSIEnable(SSI3_BASE);

    /* configure host IRQ line (pin PM7)*/
    GPIOPinTypeGPIOInput(GPIO_PORTM_BASE, GPIO_PIN_7);
    GPIOPadConfigSet(GPIO_PORTM_BASE, GPIO_PIN_7, GPIO_STRENGTH_2MA,
                     GPIO_PIN_TYPE_STD_WPD);
    GPIOIntTypeSet(GPIO_PORTM_BASE, GPIO_PIN_7, GPIO_RISING_EDGE);
    GPIOIntClear(GPIO_PORTM_BASE,GPIO_PIN_7);
    GPIOIntDisable(GPIO_PORTM_BASE,GPIO_PIN_7);
    IntEnable(INT_GPIOM);
    IntMasterEnable();

    /* 1 ms delay */
    SysCtlDelay( (g_ui32SysClock/(3*1000))*50 );

    /* Enable WLAN interrupt */
    CC3100_InterruptEnable();

    return NONOS_RET_OK;
}


int spi_Write(Fd_t fd, unsigned char *pBuff, int len)
{
    int len_to_return = len;
    unsigned long ulDummy;

    ASSERT_CS();

    while(len)
    {
        while(SSIDataPutNonBlocking(SSI3_BASE, (unsigned long)*pBuff) != TRUE);
        while(SSIDataGetNonBlocking(SSI3_BASE, &ulDummy) != TRUE);
        pBuff++;
        len--;
    }
    DEASSERT_CS();

    return len_to_return;
}


int spi_Read(Fd_t fd, unsigned char *pBuff, int len)
{
    int i = 0;
    unsigned long ulBuff;

    ASSERT_CS();

    for(i=0; i< len; i++)
    {
        while(SSIDataPutNonBlocking(SSI3_BASE, 0xFF) != TRUE);
        while(SSIDataGetNonBlocking(SSI3_BASE, &ulBuff) != TRUE);
        pBuff[i] = (unsigned char)ulBuff;
    }
    DEASSERT_CS();

    return len;
}
#endif /* SL_IF_TYPE_UART */
