//*****************************************************************************
//
// cli_uart.c - TM4C1294 launchpad application uart interface implementation
//
// Copyright (c) 2015 Texas Instruments Incorporated - http://www.ti.com/
// ALL RIGHTS RESERVED
//
//*****************************************************************************

//*****************************************************************************
//
//! \addtogroup CLI_UART
//! @{
//
//*****************************************************************************

#include <string.h>
#include <stdint.h>
#include <stdbool.h>
#include "inc/hw_ints.h"
#include "inc/hw_memmap.h"
#include "driverlib/gpio.h"
#include "driverlib/pin_map.h"
#include "driverlib/rom.h"
#include "driverlib/sysctl.h"
#include "driverlib/uart.h"
#include "cli_uart.h"


#define ASCII_ENTER     0x0D
#define LOOP_FOREVER    1

#ifdef _USE_CLI_
unsigned char *g_ucUARTBuffer;

int cli_have_cmd = 0;
#endif


int
CLI_Read(unsigned char *pBuff)
{
    if(pBuff == NULL)
        return -1;

#ifdef _USE_CLI_
    g_ucUARTBuffer = pBuff;


    while(LOOP_FOREVER)
    {
        *g_ucUARTBuffer = (unsigned char)UARTCharGet(UART0_BASE);

        if(*g_ucUARTBuffer == ASCII_ENTER)
        {
            *g_ucUARTBuffer = 0x00;
            break;
        }

        g_ucUARTBuffer++;
    }

    return strlen((const char *)pBuff);

#else
    return 0;
#endif
}


int
CLI_Write(unsigned char *inBuff)
{
    if(inBuff == NULL)
        return -1;

#ifdef _USE_CLI_
    unsigned short ret = 0;
    unsigned short usLength = strlen((const char *)inBuff);
    ret = usLength;

    while (usLength)
    {
        UARTCharPut(UART0_BASE, *inBuff);
        usLength--;
        inBuff++;
    }
    return (int)ret;
#else
    return 0;
#endif
}

extern uint32_t g_ui32SysClock;
void
CLI_Configure(void)
{
#ifdef _USE_CLI_

    /* Enable the UART0 and GPIOA peripherals clock. */
    SysCtlPeripheralEnable(SYSCTL_PERIPH_UART0);
    SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOA);

    /* Set GPIO A0 and A1 as UART pins. */
    GPIOPinConfigure(GPIO_PA0_U0RX);
    GPIOPinConfigure(GPIO_PA1_U0TX);
    GPIOPinTypeUART(GPIO_PORTA_BASE, GPIO_PIN_0 | GPIO_PIN_1);

    /* Configure the UART for 115,200, 8-N-1 operation. */
    UARTConfigSetExpClk(UART0_BASE, g_ui32SysClock, 115200,
                        (UART_CONFIG_WLEN_8 | UART_CONFIG_STOP_ONE |
                         UART_CONFIG_PAR_NONE));

#endif
}


//*****************************************************************************
//
// Close the Doxygen group.
//! @}
//
//*****************************************************************************
