//*****************************************************************************
//
// cli_uart.c - TM4C1294 launchpad application uart interface implementation
//
// Copyright (c) 2015 Texas Instruments Incorporated - http://www.ti.com/
// ALL RIGHTS RESERVED
//
//*****************************************************************************

#ifndef __CLI_UART_H__
#define __CLI_UART_H__


//****************************************************************************
//          Define _USE_CLI_ Macro in project's properties to use the
//          command line interface
//****************************************************************************

//*****************************************************************************
//
// If building with a C++ compiler, make all of the definitions in this header
// have a C binding.
//
//*****************************************************************************
#ifdef __cplusplus
extern "C" {
#endif

/*!
    \brief      Read the data on Application UArt channel,
                User is expected to assign memory and pass
                the pointer to buffer to the function

    \param[in]  pBuff - pointer to the read buffer
                User should allocate the memory to the buffer
                and pass the pointer to API

    \return     int - No of bytes read, -1 in case of error

    \note

    \warning
*/
extern int CLI_Read(unsigned char *pBuff);

/*!
    \brief      Configures the Application Uart

    \param[in]  none

    \return     none

    \note

    \warning
*/
extern void CLI_Configure(void);

/*!
    \brief      Write the data on Application Uart channel

    \param[in]  inBuff - Pointer to output data, string should be null
                terminated

    \return     int - No of byte sent, -1 in case of error

    \note

    \warning
*/
extern int CLI_Write(unsigned char *inBuff);

//*****************************************************************************
//
// Mark the end of the C bindings section for C++ compilers.
//
//*****************************************************************************
#ifdef    __cplusplus
}
#endif // __cplusplus

#endif // __CLI_UART_H__
