/*
 * main.c
 *
 * Copyright (c) 2015 Texas Instruments Incorporated - http://www.ti.com/
 * ALL RIGHTS RESERVED
 */

/**********************************************************************************************************************
 * Header Files
 *********************************************************************************************************************/
/* XDCtools Header files */
#include <xdc/std.h>

#include <xdc/cfg/global.h>
#include <xdc/runtime/Error.h>
#include <xdc/runtime/System.h>
#include <xdc/runtime/Types.h>

/* BIOS Header files */
#include <ti/sysbios/BIOS.h>
#include <ti/sysbios/knl/Task.h>

/* TI-RTOS Drivers Header files */
#include <ti/drivers/GPIO.h>
#include <ti/drivers/SPI.h>

/* Example/Board Header files */
#include "Board.h"

/* I/O Header file for Wi-Fi Example */
#include "io.h"

/* CLI Header File*/
#include "cli_uart.h"

/*********************************************************************************************************************
 * Exported Object Definitions
 *********************************************************************************************************************/
/* Global reference to clock frequency */
uint32_t g_ui32SysClock;

/*********************************************************************************************************************
 *  Function Definitions
 *********************************************************************************************************************/
Int main()
{
	/* Local variable to get clock frequency*/
	Types_FreqHz clockclock;

    /* Call board init functions. */
    Board_initGeneral();
    /* Initialize GPIO */
    Board_initGPIO();

    /* Initialize SPI */
    Board_initSPI();

    /* Get CPU Frequency*/
    BIOS_getCpuFreq(&clockclock);
    g_ui32SysClock = clockclock.lo;

    /*
     * Configure command line interface.
     * UART driver is not thread safe and is currently accessed by one single task
     */
    CLI_Configure();

    /* Configure interrupts and peripherals used by HTTP server task */
    IoInit();

    BIOS_start();    /* does not return */

    return(0);
}
