//*****************************************************************************
//
// nfc_spi_rtos.c
//
// Copyright (c) 2015 Texas Instruments Incorporated - http://www.ti.com/
// ALL RIGHTS RESERVED
//
//*****************************************************************************
// XDCtools Header files
#include <xdc/runtime/System.h>

#ifdef TI_RTOS
#include <ti/sysbios/BIOS.h>
#include <ti/drivers/SPI.h>
#include <ti/drivers/GPIO.h>
#include "Board.h"
#endif

#include "nfc_spi.h"

#ifndef TRUE
enum BOOL_FLAG { FALSE, TRUE };
#endif
//===============================================================

// Direct Mode disabled
uint8_t g_direct_mode = 0;

bool bSpiComplete = false;

void SPI_Callback(SPI_Handle handle, SPI_Transaction *transaction)
{
	/* Set Flag */
	bSpiComplete = true;
}

void SPI_directCommand( uint8_t command )
{
    SPI_Transaction masterTransaction;
    uint8_t DummyByte = 0x00U;
    bool transferOK;

	
	GPIO_write(Board_TRF7970A_CS, Board_TRF7970A_CS_ENABLE); // Start SPI Mode

	// set Address/Command Word Bit Distribution to command
	command = (0x80 | command);					// command
	command = (0x9f & command);					// command code
	
	masterTransaction.count = 1;
    masterTransaction.txBuf = ( Ptr )&command;
    masterTransaction.rxBuf = ( Ptr )&DummyByte;

    //System_printf("SPI_directCommand transfer start\n\n");
    //System_flush();
	transferOK = SPI_transfer( NFCInfo.hMasterSPI, &masterTransaction );
	if( !transferOK )
	{
		//System_printf("Unsuccessful master SPI_directCommand transfer\n\n");
		//System_flush();
	}
	else
	{
		//System_printf("Successful master SPI transfer\n\n");
		//System_flush();
	}

	while(false == bSpiComplete);

	bSpiComplete = false;

	GPIO_write(Board_TRF7970A_CS, Board_TRF7970A_CS_DISABLE); //Stop SPI Mode
}

//===============================================================

void SPI_rawWrite(uint8_t *pbuf, uint8_t length)
{	
    SPI_Transaction masterTransaction;
    bool transferOK;
	volatile uint8_t x;

    masterTransaction.count = length;
    masterTransaction.txBuf = ( Ptr )pbuf;
    masterTransaction.rxBuf = ( Ptr )NULL;
    GPIO_write(Board_TRF7970A_CS, Board_TRF7970A_CS_ENABLE); //Start SPI Mode

    transferOK = SPI_transfer( NFCInfo.hMasterSPI, &masterTransaction );

	while(false == bSpiComplete);

	bSpiComplete = false;
	
    if( !transferOK )
    {
        //System_printf("Unsuccessful master SPI transfer\n\n");
	}

    if(g_direct_mode == 0x00)
    {
        GPIO_write(Board_TRF7970A_CS, Board_TRF7970A_CS_DISABLE); // Stop SPI Mode
    }
}

void SPI_readCont(uint8_t *pbuf, uint8_t reg, uint8_t length)
{	
    SPI_Transaction masterTransaction;
    uint8_t DummyByte;
    bool transferOK;
	volatile int8_t x;

    GPIO_write(Board_TRF7970A_CS, Board_TRF7970A_CS_ENABLE); // Start SPI Mode

	// Address/Command Word Bit Distribution
	// address, write, single (fist 3 bits = 0)
	reg = (0x1f & reg);					// register address
	reg = (0x60 | reg); 				// address, read, continuous
    masterTransaction.count = 1;
    masterTransaction.txBuf = ( Ptr )&reg;
    masterTransaction.rxBuf = ( Ptr )&DummyByte;
	transferOK = SPI_transfer( NFCInfo.hMasterSPI, &masterTransaction );

	while(false == bSpiComplete);

	bSpiComplete = false;
	if( !transferOK )
	{
		//System_printf("Unsuccessful master SPI transfer\n");
	}

    masterTransaction.count = length;
    masterTransaction.txBuf = NULL;
    masterTransaction.rxBuf = ( Ptr )pbuf;
	transferOK = SPI_transfer( NFCInfo.hMasterSPI, &masterTransaction );

	while(false == bSpiComplete);

	bSpiComplete = false;
	if( !transferOK )
	{
		//System_printf("Unsuccessful master SPI transfer\n");
	}

	GPIO_write(Board_TRF7970A_CS, Board_TRF7970A_CS_DISABLE); // Stop SPI Mode

}

void SPI_readSingle(uint8_t *pbuf, uint8_t reg)
{			
    SPI_Transaction masterTransaction;
    uint8_t DummyByte;
    bool transferOK;
	volatile uint8_t x;

	GPIO_write(Board_TRF7970A_CS, Board_TRF7970A_CS_ENABLE); // Start SPI Mode

	// Address/Command Word Bit Distribution
	// address, write, single (fist 3 bits = 0)
	reg = (0x1f & reg);				// register address
	reg = (0x40 | reg); 			// address, read, single
	
    masterTransaction.count = 1;
    masterTransaction.txBuf = ( Ptr )&reg;
    masterTransaction.rxBuf = ( Ptr )&DummyByte;
	transferOK = SPI_transfer( NFCInfo.hMasterSPI, &masterTransaction );

	while(false == bSpiComplete);

	bSpiComplete = false;
	if( !transferOK )
	{
		//System_printf("Unsuccessful master SPI transfer\n");
	}

	DummyByte = 0;
	masterTransaction.count = 1;
    masterTransaction.txBuf = ( Ptr )&DummyByte;
    masterTransaction.rxBuf = ( Ptr )pbuf;
	transferOK = SPI_transfer( NFCInfo.hMasterSPI, &masterTransaction );

	while(false == bSpiComplete);

	bSpiComplete = false;
	if( !transferOK )
	{
		//System_printf("Unsuccessful master SPI transfer\n");
	}

	if(g_direct_mode == 0x00)
	{
		GPIO_write(Board_TRF7970A_CS, Board_TRF7970A_CS_DISABLE); // Stop SPI Mode
	}
}
/*
void SPI_setup(void)
{	
   USCI_B_SPI_masterInit(SPI_MODULE_BASE_ADDR,
						USCI_B_SPI_CLOCKSOURCE_SMCLK,
						MCLK_FREQ,
						SPI_CLK_FREQ,
						USCI_B_SPI_MSB_FIRST,
						USCI_B_SPI_PHASE_DATA_CHANGED_ONFIRST_CAPTURED_ON_NEXT,
						USCI_B_SPI_CLOCKPOLARITY_INACTIVITY_LOW
						);

    GPIO_setAsPeripheralModuleFunctionOutputPin(SPI_PORT, SPI_CLK | SPI_MOSI | SPI_MISO);

    USCI_B_SPI_enable(SPI_MODULE_BASE_ADDR);
}
*/
void SPI_writeSingle(uint8_t data, uint8_t reg)
{
    SPI_Transaction masterTransaction;
    uint8_t DummyByte;
    bool transferOK;
	volatile int8_t x;

	GPIO_write(Board_TRF7970A_CS, Board_TRF7970A_CS_ENABLE); // Start SPI Mode

	// Address/Command Word Bit Distribution
	// address, write, single (fist 3 bits = 0)
	reg = ( 0x1f & reg );				// register address

    masterTransaction.count = 1;
    masterTransaction.txBuf = ( Ptr )&reg;
    //masterTransaction.rxBuf = NULL;// ( Ptr )&DummyByte;
    masterTransaction.rxBuf = ( Ptr )&DummyByte;
	transferOK = SPI_transfer( NFCInfo.hMasterSPI, &masterTransaction );
	while(false == bSpiComplete);

	bSpiComplete = false;
	if( !transferOK )
	{
		//System_printf("Unsuccessful master SPI transfer\n");
	}

    masterTransaction.count = 1;
    masterTransaction.txBuf = ( Ptr )&data;
    //masterTransaction.rxBuf = NULL;// ( Ptr )&DummyByte;
    masterTransaction.rxBuf = ( Ptr )&DummyByte;
    //System_printf("SPI_writeSingle transfer start\n\n");
    //System_flush();
	transferOK = SPI_transfer( NFCInfo.hMasterSPI, &masterTransaction );
	while(false == bSpiComplete);

	bSpiComplete = false;
	if( !transferOK )
	{
		//System_printf("Unsuccessful master SPI_writeSingle transfer\n");
		//System_flush();
	}
	else
	{
		//System_printf("Successful master SPI_writeSingle transfer\n");
		//System_flush();
	}

	if(g_direct_mode == 0x00)
	{
		GPIO_write(Board_TRF7970A_CS, Board_TRF7970A_CS_DISABLE);
	}
}

void SPI_writeCont(uint8_t *pbuf, uint8_t reg, uint8_t length)
{
    SPI_Transaction masterTransaction;
    bool transferOK;
	volatile int8_t x[10];

	GPIO_write(Board_TRF7970A_CS, Board_TRF7970A_CS_ENABLE); // Start SPI Mode

	// Address/Command Word Bit Distribution
	// address, write, single (fist 3 bits = 0)
	reg = (0x20 | reg); 				// address, write, continuous
	reg = (0x3f & reg);					// register address

    masterTransaction.count = 1;
    masterTransaction.txBuf = ( Ptr )&reg;
    //masterTransaction.rxBuf = NULL;// ( Ptr )&DummyByte;
    masterTransaction.rxBuf = ( Ptr )&x;
    //System_printf("SPI_writeCont transfer start\n\n");
    //System_flush();
	transferOK = SPI_transfer( NFCInfo.hMasterSPI, &masterTransaction );
	while(false == bSpiComplete);

	bSpiComplete = false;
	if( !transferOK )
	{
		//System_printf("Unsuccessful master SPI_writeCont transfer\n");
		//System_flush();
	}
	else
	{
		//System_printf("Successful master SPI_writeCont transfer\n");
		//System_flush();
	}

    masterTransaction.count = length;
    masterTransaction.txBuf = ( Ptr )pbuf;
    //masterTransaction.rxBuf = NULL;
    masterTransaction.rxBuf = ( Ptr )&x;
    //System_printf("SPI_writeCont transfer start\n\n");
    //System_flush();
	transferOK = SPI_transfer( NFCInfo.hMasterSPI, &masterTransaction );
	while(false == bSpiComplete);

	bSpiComplete = false;
	if( !transferOK )
	{
		//System_printf("Unsuccessful master SPI_writeCont transfer\n");
		//System_flush();
	}
	else
	{
		//System_printf("Successful master SPI_writeCont transfer\n");
		//System_flush();
	}

	GPIO_write(Board_TRF7970A_CS, Board_TRF7970A_CS_DISABLE); // Stop SPI Mode
}

void SPI_writePacket(uint8_t *pbuf, uint8_t crc_bit, uint8_t total_length, uint8_t payload_length, bool header_enable, uint8_t broken_bits, bool bNFCAHeader)
{
    SPI_Transaction masterTransaction;
    volatile bool transferOK;
	uint8_t ui8LenLowerNibble, Reg;
	uint8_t ui8LenHigherNibble;
	uint16_t ui16TotalLength;

	ui16TotalLength = total_length;

	// Add one to the total length sent to the TRF7970A
	if(bNFCAHeader == true)
	{
		ui16TotalLength++;
	}

	ui8LenLowerNibble = (ui16TotalLength & 0x0F) << 4;
	ui8LenHigherNibble = (uint8_t) ((ui16TotalLength & 0x0FF0) >> 4);

	GPIO_write(Board_TRF7970A_CS, Board_TRF7970A_CS_ENABLE);// Start SPI Mode

	if(header_enable == TRUE)
	{
		// RESET FIFO
		Reg = 0x8F;
	    masterTransaction.count = 1;
	    masterTransaction.txBuf = ( Ptr )&Reg;
	    masterTransaction.rxBuf = NULL; //( Ptr )&DummyByte;
	    //System_printf("SPI_writePacket transfer start\n\n");
	    //System_flush();
		transferOK = SPI_transfer( NFCInfo.hMasterSPI, &masterTransaction );
		while(false == bSpiComplete);

		bSpiComplete = false;
		if( !transferOK )
		{
			//System_printf("Unsuccessful master SPI_writePacket transfer\n");
			//System_flush();
		}
		else
		{
			//System_printf("Successful master SPI_writePacket transfer\n");
			//System_flush();
		}

		// CRC COMMAND
		Reg = 0x90 | ( crc_bit & 0x01 );
	    masterTransaction.count = 1;
	    masterTransaction.txBuf = ( Ptr )&Reg;
	    masterTransaction.rxBuf = NULL; //( Ptr )&DummyByte;
		transferOK = SPI_transfer( NFCInfo.hMasterSPI, &masterTransaction );
		while(false == bSpiComplete);

		bSpiComplete = false;
		if( !transferOK )
		{
			//System_printf("Unsuccessful master SPI transfer\n");
			//System_flush();
		}

		// WRITE TO LENGTH REG
		Reg = 0x3D;
	    masterTransaction.count = 1;
	    masterTransaction.txBuf = ( Ptr )&Reg;
	    masterTransaction.rxBuf = NULL; //( Ptr )&DummyByte;
		transferOK = SPI_transfer( NFCInfo.hMasterSPI, &masterTransaction );
		while(false == bSpiComplete);

		bSpiComplete = false;
		if( !transferOK )
		{
			//System_printf("Unsuccessful master SPI transfer\n");
		}

		if(broken_bits == 0x00)
		{
			// LENGTH HIGH Nibble
            masterTransaction.count = 1;
		    masterTransaction.txBuf = ( Ptr )&ui8LenHigherNibble;
		    masterTransaction.rxBuf = NULL; //( Ptr )&DummyByte;
			transferOK = SPI_transfer( NFCInfo.hMasterSPI, &masterTransaction );
			while(false == bSpiComplete);

			bSpiComplete = false;
			if( !transferOK )
			{
				//System_printf("Unsuccessful master SPI transfer\n");
			}

			// LENGTH LOW Nibble
		    masterTransaction.count = 1;
		    masterTransaction.txBuf = ( Ptr )&ui8LenLowerNibble;
		    masterTransaction.rxBuf = NULL; //( Ptr )&DummyByte;
			transferOK = SPI_transfer( NFCInfo.hMasterSPI, &masterTransaction );
			while(false == bSpiComplete);

			bSpiComplete = false;
			if( !transferOK )
			{
				//System_printf("Unsuccessful master SPI transfer\n");
			}
		}
		else
		{
			// Broken Bit
			Reg = 0x00;
		    masterTransaction.count = 1;
		    masterTransaction.txBuf = ( Ptr )&Reg;
		    masterTransaction.rxBuf = NULL; //( Ptr )&DummyByte;
			transferOK = SPI_transfer( NFCInfo.hMasterSPI, &masterTransaction );
			while(false == bSpiComplete);

			bSpiComplete = false;
			if( !transferOK )
			{
				//System_printf("Unsuccessful master SPI transfer\n");
			}

			Reg = ( (broken_bits << 0x01) | 0x01);
		    masterTransaction.count = 1;
		    masterTransaction.txBuf = ( Ptr )&Reg;
		    masterTransaction.rxBuf = NULL; //( Ptr )&DummyByte;
			transferOK = SPI_transfer( NFCInfo.hMasterSPI, &masterTransaction );
			while(false == bSpiComplete);

			bSpiComplete = false;
			if( !transferOK )
			{
				//System_printf("Unsuccessful master SPI transfer\n");
			}
		}

		if(bNFCAHeader == true)
		{
			// Type A Header Byte
			Reg = 0xF0;
		    masterTransaction.count = 1;
		    masterTransaction.txBuf = ( Ptr )&Reg;
		    masterTransaction.rxBuf = NULL; //( Ptr )&DummyByte;
			transferOK = SPI_transfer( NFCInfo.hMasterSPI, &masterTransaction );
			while(false == bSpiComplete);

			bSpiComplete = false;
			if( !transferOK )
			{
				//System_printf("Unsuccessful master SPI transfer\n");
			}
		}
	}
	else
	{
		// Always do a continious write to the FIFO (even if there is only one byte)
		Reg = 0x3F;
	    masterTransaction.count = 1;
	    masterTransaction.txBuf = ( Ptr )&Reg;
	    masterTransaction.rxBuf = NULL; //( Ptr )&DummyByte;
		transferOK = SPI_transfer( NFCInfo.hMasterSPI, &masterTransaction );
		while(false == bSpiComplete);

		bSpiComplete = false;
		if( !transferOK )
		{
			//System_printf("Unsuccessful master SPI transfer\n");
		}
	}

    masterTransaction.count = payload_length;
    masterTransaction.txBuf = ( Ptr )pbuf;
    masterTransaction.rxBuf = NULL; //( Ptr )DummyBuffer;
	transferOK = SPI_transfer( NFCInfo.hMasterSPI, &masterTransaction );
	while(false == bSpiComplete);

	bSpiComplete = false;
	if( !transferOK )
	{
		//System_printf("Unsuccessful master SPI transfer\n");
	}

	GPIO_write(Board_TRF7970A_CS, Board_TRF7970A_CS_DISABLE); // Stop SPI Mode
}
