//*****************************************************************************
//
// t2t_app.c
//
// Copyright (c) 2015 Texas Instruments Incorporated - http://www.ti.com/
// ALL RIGHTS RESERVED
//
//*****************************************************************************

#include "tag_header.h"
#include "mcu.h"

uint16_t g_ui16T2TTlvLenRemaining;

uint8_t g_ui8T2TCurrentTlv;

tT2TStateMachine g_eT2TState;

uint8_t g_pui8T2TRxBuffer[16];

bool g_bT2TWaitForRsp;

uint8_t g_ui8T2TBlockNumber;

uint16_t g_ui16T2TNDEFBlockRemaining;

uint8_t * g_pui8T2TBuffer;

uint16_t g_ui16T2TNdefLen;

uint16_t g_ui16T2TMaxNdefLen;

uint16_t g_ui16T2TNdefIndex;

void T2T_init(uint8_t * pui8Ndef, uint16_t ui16NdefMaxSize)
{
    g_eT2TState = T2T_READ_CC;
    g_ui8T2TCurrentTlv = 0;
    g_ui16T2TTlvLenRemaining = 0;
    g_bT2TWaitForRsp = false;

    g_pui8T2TBuffer = pui8Ndef;
    g_ui16T2TMaxNdefLen = ui16NdefMaxSize;

    g_ui8T2TBlockNumber = 0;
    g_ui16T2TNDEFBlockRemaining = 0;

    g_ui16T2TNdefIndex = 0;
}

void T2T_writeNDEF(uint8_t * pui8Data, uint16_t ui16NdefLen)
{
    uint8_t ui8Offset;

    ui8Offset = 0;

    if(g_eT2TState == T2T_SELECTED_IDLE)
    {
        g_eT2TState = T2T_WRITE_NDEF;

        g_ui16T2TNdefIndex = 0;

        // Start Writing at Block 4
        g_ui8T2TBlockNumber = 0x04;

        // Lock Control TLV - Type
        g_pui8T2TBuffer[ui8Offset++] = TLV_LOCK_CONTROL;
        // Lock Control TLV - Length
        g_pui8T2TBuffer[ui8Offset++] = 0x03;
        // Lock Control TLV - Values
        g_pui8T2TBuffer[ui8Offset++] = 0xA0;
        g_pui8T2TBuffer[ui8Offset++] = 0x10;
        g_pui8T2TBuffer[ui8Offset++] = 0x44;
        // NDEF Msg TLV - Type
        g_pui8T2TBuffer[ui8Offset++] = TLV_NDEF_MSG;
        // NDEF Msg TLV - Length
        if(ui16NdefLen < 0xFF)
        {
            g_pui8T2TBuffer[ui8Offset++] = (uint8_t) ui16NdefLen;
        }

        g_ui16T2TTlvLenRemaining = (uint16_t) ui8Offset+ui16NdefLen+1;

        memcpy(&g_pui8T2TBuffer[ui8Offset],pui8Data,ui16NdefLen);

        g_pui8T2TBuffer[ui8Offset+ui16NdefLen] = LAST_TLV;
    }
}

void T2T_stateMachine(void)
{
    uint8_t ui8RxLength;
    uint8_t * pui8RxData;
    uint8_t ui8TempLength;
    char pui8LenBuffer[3];

    tNfcRwT2TConnectionStatus eT2TStatus;

    uint8_t ui8ReadIndex;

    if(g_bT2TWaitForRsp)
    {
        switch(g_eT2TState)
        {
            case T2T_READ_CC:
                eT2TStatus = NFC_RW_T2T_getReadStatus();

                if(eT2TStatus == NFC_RW_T2T_CONNECTION_READ_SUCCESS)
                {
                    NFC_RW_T2T_getPacketStatus(&pui8RxData,&ui8RxLength);

                    // Store the NDEF in a dummy buffer
                    memcpy(g_pui8T2TRxBuffer,pui8RxData,ui8RxLength);

                    // Check if NDEF compliant
                    if(pui8RxData[0] == 0xE1)
                    {
                        // Switch to the read NDEF data
                        g_eT2TState = T2T_READ_NDEF;

                        g_ui16T2TNDEFBlockRemaining = ((uint16_t) pui8RxData[2]) << 1;
                    }
                    else
                    {
                        // Switch to the read tag data
                        g_eT2TState = T2T_READ_DATA;

                        Serial_printf("T2T Not NDEF Formatted",RW_STATUS_DATA);

                        pui8LenBuffer[0] = 0x0D;
                        pui8LenBuffer[1] = 0x0A;
                        pui8LenBuffer[2] = 0x00;

                        // Print New Line
                        Serial_printf(pui8LenBuffer,RW_STATUS_DATA);

                        Serial_printf("Block ",RW_STATUS_DATA);

                        // Print Block Number
                        convertByteToAscii(g_ui8T2TBlockNumber,(uint8_t *) pui8LenBuffer);

                        Serial_printf(pui8LenBuffer,RW_STATUS_DATA);

                        Serial_printf(" ",RW_STATUS_DATA);

                        for(ui8TempLength = 0; ui8TempLength < 4; ui8TempLength++)
                        {
                            convertByteToAscii(pui8RxData[ui8TempLength],(uint8_t *) pui8LenBuffer);

                            Serial_printf("0x",RW_STATUS_DATA);

                            Serial_printf(pui8LenBuffer,RW_STATUS_DATA);

                            Serial_printf(" ",RW_STATUS_DATA);
                        }

                        pui8LenBuffer[0] = 0x0D;
                        pui8LenBuffer[1] = 0x0A;
                        pui8LenBuffer[2] = 0x00;

                        Serial_printf(pui8LenBuffer,RW_STATUS_DATA);

                        // Fixed amount
                        g_ui16T2TNDEFBlockRemaining = 12;
                    }

                    g_bT2TWaitForRsp = false;

                    // Incrment Block Number
                    g_ui8T2TBlockNumber++;

                }
                else if(eT2TStatus == NFC_RW_T2T_CONNECTION_READ_FAIL)
                {
                    g_bT2TWaitForRsp = false;
                    g_eT2TState = T2T_SELECTED_IDLE;
                }
                break;
            case T2T_READ_DATA:
                eT2TStatus = NFC_RW_T2T_getReadStatus();

                if(eT2TStatus == NFC_RW_T2T_CONNECTION_READ_SUCCESS)
                {
                    Serial_printf("Block ",RW_STATUS_DATA);

                    // Print Block Number
                    convertByteToAscii(g_ui8T2TBlockNumber,(uint8_t *) pui8LenBuffer);

                    Serial_printf(pui8LenBuffer,RW_STATUS_DATA);

                    Serial_printf(" ",RW_STATUS_DATA);

                    NFC_RW_T2T_getPacketStatus(&pui8RxData,&ui8RxLength);

                    // Store the NDEF in a dummy buffer
                    memcpy(g_pui8T2TRxBuffer,pui8RxData,ui8RxLength);

                    for(ui8TempLength = 0; ui8TempLength < 4; ui8TempLength++)
                    {
                        convertByteToAscii(pui8RxData[ui8TempLength], (uint8_t *) pui8LenBuffer);

                        Serial_printf("0x",RW_STATUS_DATA);

                        Serial_printf(pui8LenBuffer,RW_STATUS_DATA);

                        Serial_printf(" ",RW_STATUS_DATA);
                    }

                    pui8LenBuffer[0] = 0x0D;
                    pui8LenBuffer[1] = 0x0A;
                    pui8LenBuffer[2] = 0x00;

                    Serial_printf(pui8LenBuffer,RW_STATUS_DATA);

                    g_bT2TWaitForRsp = false;

                    // Incrment Block Number
                    g_ui8T2TBlockNumber++;

                    g_ui16T2TNDEFBlockRemaining--;

                    if(g_ui16T2TNDEFBlockRemaining == 0x00)
                    {
                        g_eT2TState = T2T_SELECTED_IDLE;
                    }
                }
                else if(eT2TStatus == NFC_RW_T2T_CONNECTION_READ_FAIL)
                {
                    g_bT2TWaitForRsp = false;
                    g_eT2TState = T2T_SELECTED_IDLE;
                }
                break;
            case T2T_READ_NDEF:
                eT2TStatus = NFC_RW_T2T_getReadStatus();

                if(eT2TStatus == NFC_RW_T2T_CONNECTION_READ_SUCCESS)
                {
                    NFC_RW_T2T_getPacketStatus(&pui8RxData,&ui8RxLength);

                    // Store the NDEF in a dummy buffer
                    memcpy(g_pui8T2TRxBuffer,pui8RxData,ui8RxLength);

                    ui8ReadIndex = 0x00;

                    while(ui8ReadIndex < 16 && g_eT2TState == T2T_READ_NDEF)
                    {
                        if(g_ui16T2TTlvLenRemaining > 0)
                        {
                            if(g_ui16T2TTlvLenRemaining > 16)
                            {
                                ui8ReadIndex = 16;
                                g_ui16T2TTlvLenRemaining = g_ui16T2TTlvLenRemaining - 16;
                                if(g_ui8T2TCurrentTlv == TLV_NDEF_MSG)
                                {
                                    if(g_ui16T2TMaxNdefLen > (g_ui16T2TNdefLen + 16))
                                    {
                                        memcpy(&g_pui8T2TBuffer[g_ui16T2TNdefLen],&g_pui8T2TRxBuffer[0],(uint16_t) 16);
                                        g_ui16T2TNdefLen = g_ui16T2TNdefLen + 16;
                                    }
                                }
                            }
                            else
                            {
                                // Set the Type Index for the next TLV
                                ui8ReadIndex = (uint8_t) (g_ui16T2TTlvLenRemaining);
                                if(g_ui8T2TCurrentTlv == TLV_NDEF_MSG)
                                {
                                    if(g_ui16T2TMaxNdefLen > (g_ui16T2TNdefLen + ui8ReadIndex))
                                    {
                                        memcpy(&g_pui8T2TBuffer[g_ui16T2TNdefLen],&g_pui8T2TRxBuffer[0],ui8ReadIndex);
                                        g_ui16T2TNdefLen = g_ui16T2TNdefLen + ui8ReadIndex;
                                    }

                                    Serial_printBuffer((char *)&g_pui8T2TBuffer[0],g_ui16T2TNdefLen,RW_PAYLOAD_DATA);
                                }
                                g_ui16T2TTlvLenRemaining = 0;
                            }
                        }

                        // Handle the next TLV
                        if(g_ui16T2TTlvLenRemaining == 0)
                        {
                            g_ui8T2TCurrentTlv = g_pui8T2TRxBuffer[ui8ReadIndex];
                            ui8ReadIndex++;

                            if(g_ui8T2TCurrentTlv == LAST_TLV)
                            {
                                g_eT2TState = T2T_SELECTED_IDLE;
                            }
                            else
                            {
                                // Get the Current TLV Length
                                if(g_pui8T2TRxBuffer[ui8ReadIndex] < 0xFF)
                                {
                                    g_ui16T2TTlvLenRemaining = g_pui8T2TRxBuffer[ui8ReadIndex];
                                    ui8ReadIndex++;
                                }
                                else
                                {
                                    // No support for Larger than 256 byte NDEF
                                }

                                if(g_ui8T2TCurrentTlv == TLV_NDEF_MSG)
                                {
                                    if(g_ui16T2TTlvLenRemaining < (uint16_t) (16 - ui8ReadIndex))
                                    {
                                        Serial_printBuffer((char *)&g_pui8T2TRxBuffer[ui8ReadIndex],g_ui16T2TTlvLenRemaining,RW_PAYLOAD_DATA);
                                    }
                                    else
                                    {
                                        g_ui16T2TNdefLen = (16-ui8ReadIndex);
                                        memcpy(g_pui8T2TBuffer,&g_pui8T2TRxBuffer[ui8ReadIndex],g_ui16T2TNdefLen);
                                    }
                                }

                                // Set the Index
                                if(g_ui16T2TTlvLenRemaining < (uint16_t) (16 - ui8ReadIndex))
                                {
                                    ui8ReadIndex = ui8ReadIndex + (uint8_t) g_ui16T2TTlvLenRemaining;
                                    g_ui16T2TTlvLenRemaining = 0;
                                }
                                else
                                {
                                    g_ui16T2TTlvLenRemaining = g_ui16T2TTlvLenRemaining - (16 - ui8ReadIndex);
                                    ui8ReadIndex = 16;
                                }
                            }
                        }
                    }

                    g_bT2TWaitForRsp = false;

                    if(g_ui8T2TBlockNumber == 0xFF)
                    {
                        g_eT2TState = T2T_SELECTED_IDLE;
                    }

                    if(g_eT2TState != T2T_SELECTED_IDLE)
                    {
                        // Incrment Block Number
                        g_ui8T2TBlockNumber = g_ui8T2TBlockNumber + 4;

                        // EOF Not Found
                        if(g_ui8T2TBlockNumber > 0x0F)
                        {
                            Serial_printf("Error: T2T EOF Not Found",RW_STATUS_DATA);

                            Serial_printf(pui8LenBuffer,RW_STATUS_DATA);

                            pui8LenBuffer[0] = 0x0D;
                            pui8LenBuffer[1] = 0x0A;
                            pui8LenBuffer[2] = 0x00;

                            Serial_printf(pui8LenBuffer,RW_STATUS_DATA);

                            g_eT2TState = T2T_SELECTED_IDLE;
                        }

                        g_ui16T2TNDEFBlockRemaining = g_ui16T2TNDEFBlockRemaining - 4;
                    }
                }
                else if(eT2TStatus == NFC_RW_T2T_CONNECTION_READ_FAIL)
                {
                    g_bT2TWaitForRsp = false;
                    g_eT2TState = T2T_SELECTED_IDLE;
                }
                break;
            case T2T_WRITE_NDEF:
                eT2TStatus = NFC_RW_T2T_getWriteStatus();

                if(eT2TStatus == NFC_RW_T2T_CONNECTION_WRITE_SUCCESS)
                {
                    g_ui8T2TBlockNumber++;
                    g_ui16T2TNdefIndex = g_ui16T2TNdefIndex + 4;
                    if(g_ui16T2TTlvLenRemaining >= 4)
                    {
                        g_ui16T2TTlvLenRemaining = g_ui16T2TTlvLenRemaining - 4;
                    }
                    else
                    {
                        g_ui16T2TTlvLenRemaining = 0;
                        g_eT2TState = T2T_SELECTED_IDLE;

                        Serial_printf("Write Successful!",RW_STATUS_DATA);

                        pui8LenBuffer[0] = 0x0D;
                        pui8LenBuffer[1] = 0x0A;
                        pui8LenBuffer[2] = 0x00;

                        Serial_printf(pui8LenBuffer,RW_STATUS_DATA);
                    }
                    g_bT2TWaitForRsp = false;
                }
                else if(eT2TStatus == NFC_RW_T2T_CONNECTION_WRITE_FAIL)
                {
                    g_bT2TWaitForRsp = false;
                    g_eT2TState = T2T_SELECTED_IDLE;
                }
                break;
            case T2T_SELECTED_IDLE:
                break;
            case T2T_SECTOR_SELECT:
                eT2TStatus = NFC_RW_T2T_getSectorSelStatus();

                if(eT2TStatus == NFC_RW_T2T_CONNECTION_SECTOR_SELECT_SUCCESS)
                {
                    g_bT2TWaitForRsp = false;
                    g_eT2TState = T2T_READ_NDEF;
                }
                else if(eT2TStatus == NFC_RW_T2T_CONNECTION_SECTOR_SELECT_FAIL)
                {
                    g_bT2TWaitForRsp = false;
                    g_eT2TState = T2T_SELECTED_IDLE;
                }
                break;
        }
    }

    // Sending a command
    if(g_bT2TWaitForRsp == false)
    {
        switch(g_eT2TState)
        {
            case T2T_READ_CC:
                g_ui8T2TBlockNumber = 0x03;

                if(NFC_RW_T2T_sendReadCmd(g_ui8T2TBlockNumber) == STATUS_SUCCESS)
                {
                    g_bT2TWaitForRsp = true;
                }

                break;
            case T2T_READ_DATA:
                if(NFC_RW_T2T_sendReadCmd(g_ui8T2TBlockNumber) == STATUS_SUCCESS)
                {
                    g_bT2TWaitForRsp = true;
                }
                break;
            case T2T_READ_NDEF:
                if(NFC_RW_T2T_sendReadCmd(g_ui8T2TBlockNumber) == STATUS_SUCCESS)
                {
                    g_bT2TWaitForRsp = true;
                }
                break;
            case T2T_WRITE_NDEF:
                if(g_ui16T2TTlvLenRemaining > 0)
                {
                    // Clear the NDEF Len for function purpose
                    if(NFC_RW_T2T_sendWriteCmd(g_ui8T2TBlockNumber, &g_pui8T2TBuffer[g_ui16T2TNdefIndex],4) == STATUS_SUCCESS)
                    {
                        g_bT2TWaitForRsp = true;
                    }
                    else
                    {
                        g_eT2TState = T2T_SELECTED_IDLE;
                    }
                }
                break;
            case T2T_SELECTED_IDLE:
                break;
            case T2T_SECTOR_SELECT:
                if(NFC_RW_T2T_sendSectorSel(1) == STATUS_SUCCESS)
                {
                    g_bT2TWaitForRsp = true;
                }
                break;
        }
    }
}

