//*****************************************************************************
//
// t4t_app.c
//
// Copyright (c) 2015 Texas Instruments Incorporated - http://www.ti.com/
// ALL RIGHTS RESERVED
//
//*****************************************************************************

#include "tag_header.h"
#include "..\nfc_inc\nfc_proto.h"

const uint8_t g_pui8DTAId[14] = {0x32, 0x4E, 0x46, 0x43, 0x2E, 0x53, 0x59, 0x53, 0x2E, 0x44, 0x44, 0x46, 0x30, 0x31};

const uint8_t g_pui8NDEFId[7] = {0xD2, 0x76, 0x00, 0x00, 0x85, 0x01, 0x01};

const uint8_t g_pui8CCFileId[2] = {0xE1, 0x03};

uint8_t g_pui8NDEFFileId[2];

uint16_t g_ui16NdefLength;
uint16_t g_ui16NdefRemainingBytes;
uint16_t g_ui16T4TNdefIndex;

bool g_bNdefReadOnly;
bool g_bNdefWriteComplete;

uint8_t g_ui8NdefMaxReadBytes;
uint8_t g_ui8NdefMaxWriteBytes;

uint8_t * g_pui8T4TBuffer;
uint8_t g_ui8TxLength;

uint16_t g_ui16T4TMaxNdefLen;

tNDEFStateMachine  g_eT4TState;

bool g_bT4WaitForRsp;

void T4T_init(uint8_t * pui8Ndef, uint16_t ui16NdefMaxSize)
{
    // Initialize Global Variables
    g_ui16NdefLength = 0;
    g_ui16NdefRemainingBytes = 0;
    g_ui16T4TNdefIndex = 0;

    g_bNdefReadOnly = false;
    g_bNdefWriteComplete = false;

    g_ui8NdefMaxReadBytes = 0;
    g_ui8NdefMaxWriteBytes = 0;

    g_pui8T4TBuffer = pui8Ndef;
    g_ui16T4TMaxNdefLen = ui16NdefMaxSize;

    g_bT4WaitForRsp = false;

    g_ui8TxLength = 0;

    g_eT4TState = T4T_APP_SEL;
}

void T4T_writeNDEF(uint8_t * pui8Data, uint16_t ui16NdefLen)
{
    if(g_eT4TState == T4T_SELECTED_IDLE)
    {
        g_eT4TState = T4T_WRITE_NDEF;

        g_ui16NdefRemainingBytes = ui16NdefLen;

        g_ui16T4TNdefIndex = 0x00;

        memcpy(g_pui8T4TBuffer,pui8Data,ui16NdefLen);
    }
}

void T4T_readNDEF(uint8_t * pui8Data, uint16_t ui16NdefLen)
{
    if(g_eT4TState == T4T_SELECTED_IDLE)
    {
        memcpy(pui8Data, g_pui8T4TBuffer, ui16NdefLen);
    }
}

tNDEFStateMachine T4T_getState(void)
{
    return g_eT4TState;
}

void T4T_stateMachine(void)
{
    tISO7816ConnectionStatus sISO7816Status;
    uint16_t ui16RxLength;
    uint8_t * pui8RxData;
    uint8_t pui8LenBuffer[3];

    // Waiting to get a response for one of the states
    if(g_bT4WaitForRsp)
    {
        switch(g_eT4TState)
        {
            case T4T_APP_SEL:
                sISO7816Status = ISO_7816_4_getSelectStatus();

                if(sISO7816Status == ISO_7816_4_CONNECTION_SELECT_SUCCESS)
                {
                    g_bT4WaitForRsp = false;

                    // Go Select the Capability Container
                    g_eT4TState = T4T_SEL_CC;
                }
                else if(sISO7816Status == ISO_7816_4_CONNECTION_SELECT_FAIL)
                {
                    g_bT4WaitForRsp = false;
                    g_eT4TState = T4T_ERROR;

                    Serial_printfLine("No NDEF Support");
                }
                break;
            case T4T_ERROR:
                Serial_printfLine("T4T Error");
                break;
            case T4T_SEL_CC:
                sISO7816Status = ISO_7816_4_getSelectStatus();

                if(sISO7816Status == ISO_7816_4_CONNECTION_SELECT_SUCCESS)
                {
                    g_bT4WaitForRsp = false;

                    // Go Read the Capability Container
                    g_eT4TState = T4T_READ_CC;
                }
                else if(sISO7816Status == ISO_7816_4_CONNECTION_SELECT_FAIL)
                {
                    g_bT4WaitForRsp = false;
                    g_eT4TState = T4T_SELECTED_IDLE;
                }
                break;
            case T4T_READ_CC:
                sISO7816Status = ISO_7816_4_getReadStatus();

                if(sISO7816Status == ISO_7816_4_CONNECTION_READ_SUCCESS)
                {
                    ISO_7816_4_getPacketStatus(&pui8RxData,&ui16RxLength);

                    // MLe
                    g_ui8NdefMaxReadBytes = pui8RxData[4];
                    // MLc
                    g_ui8NdefMaxWriteBytes = pui8RxData[6];

                    // NDEF File ID
                    g_pui8NDEFFileId[0] = pui8RxData[9];
                    g_pui8NDEFFileId[1] = pui8RxData[10];

                    // Check NDEF Read-only flag
                    if(pui8RxData[14] > 0x00)
                    {
                        g_bNdefReadOnly = true;
                    }
                    else
                    {
                        g_bNdefReadOnly = false;
                    }

                    g_bT4WaitForRsp = false;

                    // Check if version number is 3.0
                    if(pui8RxData[2] == 0x30)
                    {
                        g_eT4TState = T4T_SELECTED_IDLE;
                    }
                    // Check if version number is 2.2
                    else if(pui8RxData[2] == 0x20 || pui8RxData[2] == 0x22)
                    {
                        // Go Select the NDEF File
                        g_eT4TState = T4T_SEL_NDEF_FILE;
                    }
                    else
                    {
                        g_eT4TState = T4T_SELECTED_IDLE;
                    }
                }
                else if(sISO7816Status == ISO_7816_4_CONNECTION_READ_FAIL)
                {
                    g_bT4WaitForRsp = false;
                    g_eT4TState = T4T_SELECTED_IDLE;
                }
                break;
            case T4T_SEL_NDEF_FILE:
                sISO7816Status = ISO_7816_4_getSelectStatus();

                if(sISO7816Status == ISO_7816_4_CONNECTION_SELECT_SUCCESS)
                {
                    g_bT4WaitForRsp = false;

                    // Go Read the NDEF File length
                    g_eT4TState = T4T_READ_NDEF_LEN;
                }
                else if(sISO7816Status == ISO_7816_4_CONNECTION_SELECT_FAIL)
                {
                    g_bT4WaitForRsp = false;
                    g_eT4TState = T4T_SELECTED_IDLE;
                }
                break;
            case T4T_READ_NDEF_LEN:
                sISO7816Status = ISO_7816_4_getReadStatus();

                if(sISO7816Status == ISO_7816_4_CONNECTION_READ_SUCCESS)
                {
                    ISO_7816_4_getPacketStatus(&pui8RxData,&ui16RxLength);

                    g_ui16NdefLength = (uint16_t) (pui8RxData[0] << 8) + (uint16_t) pui8RxData[1];

                    g_ui16NdefRemainingBytes = g_ui16NdefLength;

                    // Reset Index
                    g_ui16T4TNdefIndex = 0x00;

                    g_bT4WaitForRsp = false;

                    // Depending on the test

                    // Write Tests

                    // Read Tests

                    if(g_ui16NdefRemainingBytes > 0)
                    {
                        // Go Read the NDEF message
                        g_eT4TState = T4T_READ_NDEF;
                    }
                    else
                    {
                        // When no remaining bytes we are done
                        g_eT4TState = T4T_SELECTED_IDLE;
                    }

                }
                else if(sISO7816Status == ISO_7816_4_CONNECTION_READ_FAIL)
                {
                    g_bT4WaitForRsp = false;
                    g_eT4TState = T4T_SELECTED_IDLE;
                }
                break;
            case T4T_READ_NDEF:
                sISO7816Status = ISO_7816_4_getReadStatus();

                if(sISO7816Status == ISO_7816_4_CONNECTION_READ_SUCCESS)
                {
                    ISO_7816_4_getPacketStatus(&pui8RxData,&ui16RxLength);

                    ui16RxLength =  ui16RxLength - 2;

                    // Store the NDEF in a dummy buffer
                    memcpy(g_pui8T4TBuffer,pui8RxData,ui16RxLength);

                    Serial_printBuffer((char *)g_pui8T4TBuffer,ui16RxLength,RW_PAYLOAD_DATA);

                    // Update remaining bytes
                    g_ui16NdefRemainingBytes = g_ui16NdefRemainingBytes - ui16RxLength;

                    // Increase Store index
                    g_ui16T4TNdefIndex = g_ui16T4TNdefIndex + ui16RxLength;

                    g_bT4WaitForRsp = false;

                    // When completed go to the idle state

                    // Once we finish the NDEF message we are done
                    if(g_ui16NdefRemainingBytes == 0)
                    {
                        g_eT4TState = T4T_SELECTED_IDLE;
                    }
                }
                else if(sISO7816Status == ISO_7816_4_CONNECTION_READ_FAIL)
                {
                    g_bT4WaitForRsp = false;
                    g_eT4TState = T4T_SELECTED_IDLE;
                }
                break;
            case T4T_SELECTED_IDLE:
                break;
            case T4T_WRITE_NDEF:
                sISO7816Status = ISO_7816_4_getWriteStatus();

                if(sISO7816Status == ISO_7816_4_CONNECTION_WRITE_SUCCESS)
                {
                    if(g_ui16NdefRemainingBytes == 0x00)
                    {
                        g_eT4TState = T4T_SELECTED_IDLE;

                        Serial_printf("Write Successful!",RW_STATUS_DATA);

                        pui8LenBuffer[0] = 0x0D;
                        pui8LenBuffer[1] = 0x0A;
                        pui8LenBuffer[2] = 0x00;

                        Serial_printf((char *)pui8LenBuffer,RW_STATUS_DATA);
                    }
                    // Done with the test
                    g_bT4WaitForRsp = false;
                }
                else if(sISO7816Status == ISO_7816_4_CONNECTION_WRITE_FAIL)
                {
                    g_bT4WaitForRsp = false;
                    g_eT4TState = T4T_SELECTED_IDLE;
                }
                break;
            default:
                break;
        }
    }

    // Sending a command
    if(g_bT4WaitForRsp == false)
    {
        switch(g_eT4TState)
        {
            case T4T_APP_SEL:
                if(ISO_7816_4_sendSelectApplication(g_pui8NDEFId,7) == STATUS_SUCCESS)
                {
                    g_bT4WaitForRsp = true;
                }
                break;
            case T4T_ERROR:
                NFC_RW_triggerRWProtocolError();
                break;
            case T4T_SEL_CC:
                if(ISO_7816_4_sendSelectFile(g_pui8CCFileId,2) == STATUS_SUCCESS)
                {
                    g_bT4WaitForRsp = true;
                }
                break;
            case T4T_READ_CC:
                if(ISO_7816_4_sendReadFile(0,0x0F) == STATUS_SUCCESS)
                {
                    g_bT4WaitForRsp = true;
                }
                break;
            case T4T_SEL_NDEF_FILE:
                if(ISO_7816_4_sendSelectFile(g_pui8NDEFFileId,2) == STATUS_SUCCESS)
                {
                    g_bT4WaitForRsp = true;
                }
                break;
            case T4T_READ_NDEF_LEN:
                if(ISO_7816_4_sendReadFile(0,0x02) == STATUS_SUCCESS)
                {
                    g_bT4WaitForRsp = true;
                }
                break;
            case T4T_READ_NDEF:
                if(g_ui16NdefRemainingBytes > g_ui8NdefMaxReadBytes)
                {
                    if(ISO_7816_4_sendReadFile(g_ui16T4TNdefIndex+2,g_ui8NdefMaxReadBytes) == STATUS_SUCCESS)
                    {
                        g_bT4WaitForRsp = true;
                    }
                }
                else
                {
                    if(ISO_7816_4_sendReadFile(g_ui16T4TNdefIndex+2,(uint8_t) g_ui16NdefRemainingBytes) == STATUS_SUCCESS)
                    {
                        g_bT4WaitForRsp = true;
                    }
                }
                break;
            case T4T_SELECTED_IDLE:
                break;
            case T4T_WRITE_NDEF:
                // If the number of bytes remaining to be written is less than the max number of bytes
                if(g_ui16NdefRemainingBytes > 0)
                {
                    if(g_ui16NdefRemainingBytes > (uint16_t) g_ui8NdefMaxWriteBytes)
                    {

                        // Clear the NDEF Len for function purpose
                        if(ISO_7816_4_sendWriteFile(g_ui16T4TNdefIndex, g_ui8NdefMaxWriteBytes,&g_pui8T4TBuffer[g_ui16T4TNdefIndex]) == STATUS_SUCCESS)
                        {
                            g_bT4WaitForRsp = true;
                            g_ui16NdefRemainingBytes = g_ui16NdefRemainingBytes - (uint16_t) g_ui8NdefMaxWriteBytes;
                        }
                    }
                    else
                    {
                        // Clear the NDEF Len for function purpose
                        if(ISO_7816_4_sendWriteFile(g_ui16T4TNdefIndex,(uint8_t) g_ui16NdefRemainingBytes,&g_pui8T4TBuffer[g_ui16T4TNdefIndex]) == STATUS_SUCCESS)
                        {
                            g_bT4WaitForRsp = true;
                            g_ui16NdefRemainingBytes = 0;
                        }
                    }

                }
                else
                {
                    g_eT4TState = T4T_SELECTED_IDLE;
                }


                break;
            default:
                break;
        }
    }

}
