//*****************************************************************************
//
// t5t_app.c
//
// Copyright (c) 2015 Texas Instruments Incorporated - http://www.ti.com/
// ALL RIGHTS RESERVED
//
//*****************************************************************************

#include "tag_header.h"
#include "mcu.h"

tT5TStateMachine g_eT5TState;

static uint16_t g_ui16T5TBlockNumber;
static uint16_t g_ui16T5TBlockCount;
static uint8_t g_pui8T5TRxBuf[30];

bool g_bT5TWaitForRsp;

uint8_t * g_pui8T5TBuffer;

uint16_t g_ui16T5TNdefLen;

uint16_t g_ui16T5TMaxNdefLen;

uint16_t g_ui16T5TNdefIndex;

uint16_t g_ui16T5TTlvLenRemaining;

uint8_t g_ui8T5TCurrentTlv;

void T5T_init(uint8_t * pui8Ndef, uint16_t ui16NdefMaxSize)
{
    g_eT5TState = T5T_INVENTORY;
    g_ui16T5TBlockNumber = 0;
    g_ui16T5TBlockCount = 0;
    g_bT5TWaitForRsp = false;

    g_pui8T5TBuffer = pui8Ndef;

    g_ui16T5TMaxNdefLen = ui16NdefMaxSize;

    g_ui16T5TNdefIndex = 0;

    g_ui16T5TNdefLen = 0;

    g_ui16T5TTlvLenRemaining = 0;

    g_ui8T5TCurrentTlv = 0;
}

void T5T_writeNDEF(uint8_t * pui8Data, uint16_t ui16NdefLen)
{
    uint8_t ui8Offset;

    ui8Offset = 0;

    if(g_eT5TState == T5T_SELECTED_IDLE)
    {
        g_eT5TState = T5T_WRITE_NDEF;

        g_ui16T5TNdefIndex = 0;

        // Start Writing at Block 1
        g_ui16T5TBlockNumber = 0x01;

        // NDEF Msg TLV - Type
        g_pui8T5TBuffer[ui8Offset++] = TLV_NDEF_MSG;
        // NDEF Msg TLV - Length
        if(ui16NdefLen < 0xFF)
        {
            g_pui8T5TBuffer[ui8Offset++] = (uint8_t) ui16NdefLen;
        }

        g_ui16T5TTlvLenRemaining = (uint16_t) ui8Offset+ui16NdefLen+1;

        memcpy(&g_pui8T5TBuffer[ui8Offset],pui8Data,ui16NdefLen);

        g_pui8T5TBuffer[ui8Offset+ui16NdefLen] = LAST_TLV;
    }
}

void T5T_stateMachine(void)
{
    uint8_t ui8RxLength;
    uint8_t * pui8RxData;
    tNfcRwT5TConnectionStatus eT5TStatus;
    char pui8LenBuffer[5];
    uint8_t ui8Temp;
    uint8_t ui8ReadIndex;

    // Waiting for a response
    if(g_bT5TWaitForRsp == true)
    {
        switch(g_eT5TState)
        {
            case T5T_INVENTORY:
                eT5TStatus = NFC_RW_T5T_getInventoryStatus();
                if(eT5TStatus == NFC_RW_T5T_CONNECTION_INVENTORY_SUCCESS)
                {
                    g_eT5TState = T5T_GET_SYS_INFO_EXT;
                    g_bT5TWaitForRsp = false;
                }
                else
                {
                    g_eT5TState = T5T_SELECTED_IDLE;
                    g_bT5TWaitForRsp = false;
                }
                break;
            case T5T_GET_SYS_INFO_EXT:
                eT5TStatus = NFC_RW_T5T_getGetSysInfoStatus();
                if(eT5TStatus == NFC_RW_T5T_CONNECTION_GET_SYS_INFO_SUCCESS)
                {
                    NFC_RW_T5T_getPacketStatus(&pui8RxData,&ui8RxLength);

                    memcpy(g_pui8T5TRxBuf,pui8RxData,ui8RxLength);

                    g_eT5TState = T5T_READ_CC;
                    g_bT5TWaitForRsp = false;

                    g_ui16T5TBlockCount = NFC_RW_T5T_getVICCBlockCount();
                }
                else if(eT5TStatus == NFC_RW_T5T_CONNECTION_GET_SYS_INFO_FAIL)
                {
                    g_eT5TState = T5T_GET_SYS_INFO_1;
                    g_bT5TWaitForRsp = false;
                }
                break;
            case T5T_GET_SYS_INFO_1:
                eT5TStatus = NFC_RW_T5T_getGetSysInfoStatus();
                if(eT5TStatus == NFC_RW_T5T_CONNECTION_GET_SYS_INFO_SUCCESS)
                {
                    NFC_RW_T5T_getPacketStatus(&pui8RxData,&ui8RxLength);

                    memcpy(g_pui8T5TRxBuf,pui8RxData,ui8RxLength);

                    g_eT5TState = T5T_READ_CC;
                    g_bT5TWaitForRsp = false;

                    g_ui16T5TBlockCount = NFC_RW_T5T_getVICCBlockCount();
                }
                else if(eT5TStatus == NFC_RW_T5T_CONNECTION_GET_SYS_INFO_FAIL)
                {
                    g_eT5TState = T5T_GET_SYS_INFO_2;
                    g_bT5TWaitForRsp = false;
                }
                break;
            case T5T_GET_SYS_INFO_2:
                eT5TStatus = NFC_RW_T5T_getGetSysInfoStatus();
                if(eT5TStatus == NFC_RW_T5T_CONNECTION_GET_SYS_INFO_SUCCESS)
                {
                    g_eT5TState = T5T_READ_CC;
                    g_bT5TWaitForRsp = false;

                    g_ui16T5TBlockCount = NFC_RW_T5T_getVICCBlockCount();
                }
                else if(eT5TStatus == NFC_RW_T5T_CONNECTION_GET_SYS_INFO_FAIL)
                {
                    g_eT5TState = T5T_READ_CC;
                    g_bT5TWaitForRsp = false;

                    g_ui16T5TBlockCount = 0x0A;
                }
                break;
            case T5T_READ_CC:
                eT5TStatus = NFC_RW_T5T_getReadSingleStatus();
                if(eT5TStatus == NFC_RW_T5T_CONNECTION_READ_SINGLE_SUCCESS)
                {
                    NFC_RW_T5T_getPacketStatus(&pui8RxData,&ui8RxLength);

                    memcpy(g_pui8T5TRxBuf,pui8RxData+1,ui8RxLength-1);

                    // Check if NDEF message formatted
                    if(g_pui8T5TRxBuf[0] == 0xE1)
                    {
                        // Switch to the read NDEF data
                        g_eT5TState = T5T_READ_NDEF;
                    }
                    else
                    {
                        // Switch to the read NDEF data
                        g_eT5TState = T5T_READ_DATA;

                        Serial_printf("T5T Not NDEF Formatted",RW_STATUS_DATA);

                        pui8LenBuffer[0] = 0x0D;
                        pui8LenBuffer[1] = 0x0A;
                        pui8LenBuffer[2] = 0x00;

                        // Print New Line
                        Serial_printf(pui8LenBuffer,RW_STATUS_DATA);

                        Serial_printf("Block ",RW_STATUS_DATA);

                        // Print Block Number
                        convertWordToAscii(g_ui16T5TBlockNumber,(uint8_t *) pui8LenBuffer);

                        Serial_printf(pui8LenBuffer,RW_STATUS_DATA);

                        Serial_printf(" ",RW_STATUS_DATA);

                        for(ui8Temp = 0; ui8Temp < (ui8RxLength-1); ui8Temp++)
                        {
                            convertByteToAscii(g_pui8T5TRxBuf[ui8Temp],(uint8_t *) pui8LenBuffer);

                            Serial_printf("0x",RW_STATUS_DATA);

                            Serial_printf(pui8LenBuffer,RW_STATUS_DATA);

                            Serial_printf(" ",RW_STATUS_DATA);
                        }

                        pui8LenBuffer[0] = 0x0D;
                        pui8LenBuffer[1] = 0x0A;
                        pui8LenBuffer[2] = 0x00;

                        // Print New Line
                        Serial_printf(pui8LenBuffer,RW_STATUS_DATA);
                    }

                    // Incrment Block Number
                    g_ui16T5TBlockNumber++;
                    g_bT5TWaitForRsp = false;
                }
                else if(eT5TStatus == NFC_RW_T5T_CONNECTION_READ_SINGLE_FAIL)
                {
                    g_eT5TState = T5T_SELECTED_IDLE;
                    g_bT5TWaitForRsp = false;
                }
                break;
            case T5T_READ_DATA:
                eT5TStatus = NFC_RW_T5T_getReadSingleStatus();
                if(eT5TStatus == NFC_RW_T5T_CONNECTION_READ_SINGLE_SUCCESS)
                {
                    NFC_RW_T5T_getPacketStatus(&pui8RxData,&ui8RxLength);

                    memcpy(g_pui8T5TRxBuf,pui8RxData+1,ui8RxLength-1);

                    Serial_printf("Block ",RW_STATUS_DATA);

                    // Print Block Number
                    convertWordToAscii(g_ui16T5TBlockNumber,(uint8_t *) pui8LenBuffer);

                    Serial_printf(pui8LenBuffer,RW_STATUS_DATA);

                    Serial_printf(" ",RW_STATUS_DATA);

                    for(ui8Temp = 0; ui8Temp < (ui8RxLength-1); ui8Temp++)
                    {
                        convertByteToAscii(g_pui8T5TRxBuf[ui8Temp],(uint8_t *) pui8LenBuffer);

                        Serial_printf("0x",RW_STATUS_DATA);

                        Serial_printf(pui8LenBuffer,RW_STATUS_DATA);

                        Serial_printf(" ",RW_STATUS_DATA);
                    }

                    pui8LenBuffer[0] = 0x0D;
                    pui8LenBuffer[1] = 0x0A;
                    pui8LenBuffer[2] = 0x00;

                    // Print New Line
                    Serial_printf(pui8LenBuffer,RW_STATUS_DATA);

                    g_bT5TWaitForRsp = false;
                    g_ui16T5TBlockNumber++;

                    if(g_ui16T5TBlockNumber > g_ui16T5TBlockCount)
                    {
                        g_eT5TState = T5T_SELECTED_IDLE;
                    }
                }
                else if(eT5TStatus == NFC_RW_T5T_CONNECTION_READ_SINGLE_FAIL)
                {
                    g_eT5TState = T5T_SELECTED_IDLE;
                    g_bT5TWaitForRsp = false;
                }
                break;
            case T5T_READ_NDEF:
                eT5TStatus = NFC_RW_T5T_getReadSingleStatus();
                if(eT5TStatus == NFC_RW_T5T_CONNECTION_READ_SINGLE_SUCCESS)
                {
                    NFC_RW_T5T_getPacketStatus(&pui8RxData,&ui8RxLength);

                    memcpy(g_pui8T5TRxBuf,pui8RxData+1,ui8RxLength-1);

                    ui8RxLength = ui8RxLength - 1;

                    ui8ReadIndex = 0x00;

                    while(ui8ReadIndex < ui8RxLength && g_eT5TState == T5T_READ_NDEF)
                    {
                        if(g_ui16T5TTlvLenRemaining > 0)
                        {
                            if(g_ui16T5TTlvLenRemaining > (uint16_t) ui8RxLength)
                            {
                                ui8ReadIndex = ui8RxLength;
                                g_ui16T5TTlvLenRemaining = g_ui16T5TTlvLenRemaining - ui8RxLength;
                                if(g_ui8T5TCurrentTlv == TLV_NDEF_MSG)
                                {
                                    if(g_ui16T5TMaxNdefLen > (g_ui16T5TNdefLen + ui8RxLength))
                                    {
                                        memcpy(&g_pui8T5TBuffer[g_ui16T5TNdefLen],&g_pui8T5TRxBuf[0],(uint16_t) ui8RxLength);
                                        g_ui16T5TNdefLen = g_ui16T5TNdefLen + ui8RxLength;
                                    }
                                }
                            }
                            else
                            {
                                // Set the Type Index for the next TLV
                                ui8ReadIndex = (uint8_t) (g_ui16T5TTlvLenRemaining);
                                if(g_ui8T5TCurrentTlv == TLV_NDEF_MSG)
                                {
                                    if(g_ui16T5TMaxNdefLen > (g_ui16T5TNdefLen + ui8ReadIndex))
                                    {
                                        memcpy(&g_pui8T5TBuffer[g_ui16T5TNdefLen],&g_pui8T5TRxBuf[0],ui8ReadIndex);
                                        g_ui16T5TNdefLen = g_ui16T5TNdefLen + ui8ReadIndex;
                                    }

                                    Serial_printBuffer((char *)&g_pui8T5TBuffer[0],g_ui16T5TNdefLen,RW_PAYLOAD_DATA);
                                }
                                g_ui16T5TTlvLenRemaining = 0;
                            }
                        }

                        // Handle the next TLV
                        if(g_ui16T5TTlvLenRemaining == 0)
                        {
                            g_ui8T5TCurrentTlv = g_pui8T5TRxBuf[ui8ReadIndex];
                            ui8ReadIndex++;

                            if(g_ui8T5TCurrentTlv == LAST_TLV)
                            {
                                g_eT5TState = T5T_SELECTED_IDLE;
                            }
                            else
                            {
                                // Get the Current TLV Length
                                if(g_pui8T5TRxBuf[ui8ReadIndex] < 0xFF)
                                {
                                    g_ui16T5TTlvLenRemaining = g_pui8T5TRxBuf[ui8ReadIndex];
                                    ui8ReadIndex++;
                                }
                                else
                                {
                                    // No support for Larger than 256 byte NDEF
                                }

                                if(g_ui8T5TCurrentTlv == TLV_NDEF_MSG)
                                {
                                    if(g_ui16T5TTlvLenRemaining < (uint16_t) (ui8RxLength - ui8ReadIndex))
                                    {
                                        Serial_printBuffer((char *)&g_pui8T5TRxBuf[ui8ReadIndex],g_ui16T5TTlvLenRemaining,RW_PAYLOAD_DATA);
                                    }
                                    else
                                    {
                                        g_ui16T5TNdefLen = (ui8RxLength-ui8ReadIndex);
                                        memcpy(g_pui8T5TBuffer,&g_pui8T5TRxBuf[ui8ReadIndex],g_ui16T5TNdefLen);
                                    }
                                }

                                // Set the Index
                                if(g_ui16T5TTlvLenRemaining < (uint16_t) (ui8RxLength - ui8ReadIndex))
                                {
                                    ui8ReadIndex = ui8ReadIndex + (uint8_t) g_ui16T5TTlvLenRemaining;
                                    g_ui16T5TTlvLenRemaining = 0;
                                }
                                else
                                {
                                    g_ui16T5TTlvLenRemaining = g_ui16T5TTlvLenRemaining - (ui8RxLength - ui8ReadIndex);
                                    ui8ReadIndex = ui8RxLength;
                                }
                            }
                        }
                    }

                    if(g_eT5TState != T5T_SELECTED_IDLE)
                    {
                        // Incrment Block Number
                        g_ui16T5TBlockNumber++;

                        // EOF Not Found
                        if(g_ui16T5TBlockNumber > g_ui16T5TBlockCount)
                        {
                            Serial_printf("Error: T5T EOF Not Found",RW_STATUS_DATA);

                            Serial_printf(pui8LenBuffer,RW_STATUS_DATA);

                            pui8LenBuffer[0] = 0x0D;
                            pui8LenBuffer[1] = 0x0A;
                            pui8LenBuffer[2] = 0x00;

                            Serial_printf(pui8LenBuffer,RW_STATUS_DATA);

                            g_eT5TState = T5T_SELECTED_IDLE;
                        }
                    }

                    g_bT5TWaitForRsp = false;
                }
                else if(eT5TStatus == NFC_RW_T5T_CONNECTION_READ_SINGLE_FAIL)
                {
                    g_eT5TState = T5T_SELECTED_IDLE;
                    g_bT5TWaitForRsp = false;
                }
                break;
            case T5T_WRITE_NDEF:
                eT5TStatus = NFC_RW_T5T_getWriteSingleStatus();

                if(eT5TStatus == NFC_RW_T5T_CONNECTION_WRITE_SINGLE_SUCCESS)
                {
                    g_ui16T5TBlockNumber++;
                    g_ui16T5TNdefIndex = g_ui16T5TNdefIndex + 4;
                    if(g_ui16T5TTlvLenRemaining >= 4)
                    {
                        g_ui16T5TTlvLenRemaining = g_ui16T5TTlvLenRemaining - 4;
                    }
                    else
                    {
                        g_ui16T5TTlvLenRemaining = 0;
                        g_eT5TState = T5T_SELECTED_IDLE;

                        Serial_printf("Write Successful!",RW_STATUS_DATA);

                        pui8LenBuffer[0] = 0x0D;
                        pui8LenBuffer[1] = 0x0A;
                        pui8LenBuffer[2] = 0x00;

                        Serial_printf(pui8LenBuffer,RW_STATUS_DATA);
                    }
                    g_bT5TWaitForRsp = false;
                }
                else if(eT5TStatus == NFC_RW_T5T_CONNECTION_WRITE_SINGLE_FAIL)
                {
                    g_bT5TWaitForRsp = false;
                    g_eT5TState = T5T_SELECTED_IDLE;
                }
                break;
            case T5T_SELECTED_IDLE:
                g_bT5TWaitForRsp = false;
                break;
        }
    }

    // Sending a command
    if(g_bT5TWaitForRsp == false)
    {
        switch(g_eT5TState)
        {
            case T5T_INVENTORY:
                if(NFC_RW_T5T_sendInventoryCmd(0x26,0x00,false) == STATUS_SUCCESS)
                {
                    g_bT5TWaitForRsp = true;
                }
                break;
            case T5T_GET_SYS_INFO_EXT:
                if(NFC_RW_T5T_sendGetSysInfoCmd(0x02,0x3B) == STATUS_SUCCESS)
                {
                    g_bT5TWaitForRsp = true;
                }
                break;
            case T5T_GET_SYS_INFO_1:
                if(NFC_RW_T5T_sendGetSysInfoCmd(0x0A,0x2B) == STATUS_SUCCESS)
                {
                    g_bT5TWaitForRsp = true;
                }
                break;
            case T5T_GET_SYS_INFO_2:
                if(NFC_RW_T5T_sendGetSysInfoCmd(0x02,0x2B) == STATUS_SUCCESS)
                {
                    g_bT5TWaitForRsp = true;
                }
                break;
            case T5T_READ_CC:
                if(NFC_RW_T5T_sendReadSingleCmd(0x22,g_ui16T5TBlockNumber) == STATUS_SUCCESS)
                {
                    g_bT5TWaitForRsp = true;
                }
                break;
            case T5T_READ_DATA:
                if(NFC_RW_T5T_sendReadSingleCmd(0x22,g_ui16T5TBlockNumber) == STATUS_SUCCESS)
                {
                    g_bT5TWaitForRsp = true;
                }
                break;
            case T5T_READ_NDEF:
                if(NFC_RW_T5T_sendReadSingleCmd(0x22,g_ui16T5TBlockNumber) == STATUS_SUCCESS)
                {
                    g_bT5TWaitForRsp = true;
                }
                break;
            case T5T_WRITE_NDEF:
                if(g_ui16T5TTlvLenRemaining > 0)
                {
                    if(NFC_RW_T5T_sendWriteSingleCmd(T5T_REQ_FLAG_HIGH_DATA|T5T_REQ_FLAG_OPTION,g_ui16T5TBlockNumber,&g_pui8T5TBuffer[g_ui16T5TNdefIndex],4) == STATUS_SUCCESS)
                    {
                        g_bT5TWaitForRsp = true;
                    }
                    else
                    {
                        g_eT5TState = T5T_SELECTED_IDLE;
                    }
                }
                break;
            case T5T_SELECTED_IDLE:
                break;
        }
    }
}
