//*****************************************************************************
//
// nfc_utils.c
//
// Copyright (c) 2015 Texas Instruments Incorporated - http://www.ti.com/
// ALL RIGHTS RESERVED
//
//*****************************************************************************

#include <stdbool.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <stdint.h>

// XDCtools Header files
#ifdef TI_RTOS
#include <xdc/runtime/System.h>
#endif

#include "nfc_controller.h"
#include "ndef_image.h"
#include "tag_header.h"

#include "..\nfc_inc\nfc_proto.h"
#include "..\nfc_inc\nfc_define.h"
#include "..\nfc_inc\nfc_externs.h"

bool g_bEnableAutoSDD;
bool g_bExtAmplifier;
bool g_bTRF5VSupply;
tTRF79x0_Version g_eTRFVersion;
bool g_bSupportCertification;
uint16_t g_ui16ListenTime;

#if (NFC_PEER_2_PEER_INITIATOR_ENABLED || NFC_PEER_2_PEER_TARGET_ENABLED)
	t_sNfcP2PMode g_sP2PSupportedModes;
	t_sNfcP2PCommBitrate g_sP2PSupportedTargetBitrates;
	t_sNfcP2PCommBitrate g_sP2PSupportedInitiatorBitrates;
	t_sNfcDEP_P2PSetup g_sP2PSetupOptions;
	uint8_t g_ui8NfcDepInitiatorDID;
#endif

#if NFC_CARD_EMULATION_ENABLED
	t_sNfcCEMode g_sCESupportedModes;
#endif

#if NFC_READER_WRITER_ENABLED
	t_sNfcRWMode g_sRWSupportedModes;
	t_sNfcRWCommBitrate g_sRWSupportedBitrates;
	t_sIsoDEP_RWSetup g_sRWSetupOptions;
	uint8_t g_ui8IsoDepInitiatorDID;
#endif
//*****************************************************************************
//
//! NFC_configuration - Handles the initial NFC configuration.
//!
//!	Setup all NFC Mode Parameters.
//!
//! Current modes enabled: Card Emulation
//! Current modes supported: Card Emulationa and Peer 2 Peer
//! Reader/Writer is NOT supported yet.
//!
//*****************************************************************************

void NFC_configuration(void)
{
#if (NFC_PEER_2_PEER_INITIATOR_ENABLED || NFC_PEER_2_PEER_TARGET_ENABLED)
	g_sP2PSupportedModes.ui8byte = 0x00;
	g_sP2PSupportedTargetBitrates.ui8byte = 0x00;
	g_sP2PSupportedInitiatorBitrates.ui8byte = 0x00;
	g_sP2PSetupOptions.ui8byte = 0x00;
#endif

#if NFC_CARD_EMULATION_ENABLED
	g_sCESupportedModes.ui8byte = 0x00;
#endif

#if NFC_READER_WRITER_ENABLED
	g_sRWSupportedModes.ui8byte = 0x00;
	g_sRWSupportedBitrates.ui16byte = 0x0000;
	g_sRWSetupOptions.ui16byte = 0x0000;
#endif

	// Set the TRF7970 Version being used
	g_eTRFVersion = TRF7970_A;

	// External Amplifer (disconnected by default)
	g_bExtAmplifier = false;

	// Configure TRF External Amplifier for the transceiver
	TRF79x0_setExtAmplifer(g_bExtAmplifier);

	// Configure TRF Power Supply (5V = true, 3V = false)
	g_bTRF5VSupply = false;

	// Configure TRF Power Supply
	TRF79x0_setPowerSupply(g_bTRF5VSupply);

	// Milliseconds the NFC stack will be in listen mode
	g_ui16ListenTime = 500;

	// Set the time the NFC stack will be with the RF field disabled (listen mode)
	NFC_setListenTime(g_ui16ListenTime);

	// Enable (1) or disable (0) the Auto SDD Anti-collision function of the TRF7970A
	g_bEnableAutoSDD = 0;

#if (NFC_PEER_2_PEER_INITIATOR_ENABLED || NFC_PEER_2_PEER_TARGET_ENABLED)

	// Enable Peer 2 Peer Supported Modes
	g_sP2PSupportedModes.bits.bTargetEnabled = 1;
	g_sP2PSupportedModes.bits.bInitiatorEnabled = 1;

	// Set P2P Supported Bit Rates - Target mode
	g_sP2PSupportedTargetBitrates.bits.bPassive106kbps = 1;
	g_sP2PSupportedTargetBitrates.bits.bPassive212kbps = 1;
	g_sP2PSupportedTargetBitrates.bits.bPassive424kbps = 1;
	g_sP2PSupportedTargetBitrates.bits.bActive106kbps = 0;
	g_sP2PSupportedTargetBitrates.bits.bActive212kbps = 0;
	g_sP2PSupportedTargetBitrates.bits.bActive424kbps = 0;

	// Set P2P Supported Bit Rates - Initiator mode
	g_sP2PSupportedInitiatorBitrates.bits.bPassive106kbps = 0;
	g_sP2PSupportedInitiatorBitrates.bits.bPassive212kbps = 1;
	g_sP2PSupportedInitiatorBitrates.bits.bPassive424kbps = 0;
	g_sP2PSupportedInitiatorBitrates.bits.bActive106kbps = 0;
	g_sP2PSupportedInitiatorBitrates.bits.bActive212kbps = 0;
	g_sP2PSupportedInitiatorBitrates.bits.bActive424kbps = 0;

	// Certification Config Start //

	// Enable (1) or disable (0) Wave 1 NFC Forum Certification functionality
	// Note: Enabling this feature can affect interoperability with NFC Devices that are not certified.
	g_bSupportCertification = 0;

	// Allows for Customization of the DID (Device Identification) number when in initiator mode
	g_ui8NfcDepInitiatorDID = 0x00;

	// Enable LLCP
	g_sP2PSetupOptions.bits.bP2PSupportLLCP = 1;

	// Enable Loopback
	g_sP2PSetupOptions.bits.bP2PSupportLoopback = 0;

	// Specify maximum number of timeouts and protocol errors allowed before resetting
	g_sP2PSetupOptions.bits.ui3P2PMaxTimeouts = 2;
	g_sP2PSetupOptions.bits.ui3P2PMaxProtocolErrors = 2;
#endif

#if NFC_CARD_EMULATION_ENABLED

// Enable Card Emulation Supported Modes
	g_sCESupportedModes.bits.bT4TAEnabled = 0;
	g_sCESupportedModes.bits.bT4TBEnabled = 0;
	g_sCESupportedModes.bits.bT3TEnabled = 0;		// Not supported

#endif

#if NFC_READER_WRITER_ENABLED
	// Enable Reader Writer Supported Modes
	g_sRWSupportedModes.bits.bNfcA = 0;
	g_sRWSupportedModes.bits.bNfcB = 1;
	g_sRWSupportedModes.bits.bNfcF = 0;
	g_sRWSupportedModes.bits.bISO15693 = 0;

	// NFC-A Bitrates
	g_sRWSupportedBitrates.bits.bNfcA_106kbps = 0;		// Must be enabled if bNfcA is set
	g_sRWSupportedBitrates.bits.bNfcA_212kbps = 0;
	g_sRWSupportedBitrates.bits.bNfcA_424kbps = 0;
	g_sRWSupportedBitrates.bits.bNfcA_848kbps = 0;
	// NFC-B Bitrates
	g_sRWSupportedBitrates.bits.bNfcB_106kbps = 1; 		// Must be enabled if bNfcB is set
	g_sRWSupportedBitrates.bits.bNfcB_212kbps = 0;
	g_sRWSupportedBitrates.bits.bNfcB_424kbps = 0;
	g_sRWSupportedBitrates.bits.bNfcB_848kbps = 0;
	// NFC-F Bitrates
	g_sRWSupportedBitrates.bits.bNfcF_212kbps = 0;
	g_sRWSupportedBitrates.bits.bNfcF_424kbps = 0;
	// ISO15693 Bitrates
	g_sRWSupportedBitrates.bits.bISO15693_6_7kbps = 0;	// Not supported
	g_sRWSupportedBitrates.bits.bISO15693_26_48kbps = 0;

	// Default Max number of WTX 2
	g_sRWSetupOptions.bits.ui3RWMaxWTX = 2;
	// Default Max number of ACK 2
	g_sRWSetupOptions.bits.ui3RWMaxACK = 2;
	// Default Max number of NACK 2
	g_sRWSetupOptions.bits.ui3RWMaxNACK = 2;
	// Default Max number of DSL 2
	g_sRWSetupOptions.bits.ui3RWMaxDSL = 2;

	g_ui8IsoDepInitiatorDID = 0x00;
#endif

#if (NFC_PEER_2_PEER_INITIATOR_ENABLED || NFC_PEER_2_PEER_TARGET_ENABLED)
	// Configure Peer 2 Peer functions for the correct modes and communication bitrates
	NFC_P2P_configure(g_sP2PSupportedModes,g_sP2PSupportedTargetBitrates,g_sP2PSupportedInitiatorBitrates);

	// Configure NFC DEP functions including passing the DID
	NFCDEP_configure_P2P(g_sP2PSetupOptions,g_bSupportCertification,g_ui8NfcDepInitiatorDID);
#endif

#if NFC_CARD_EMULATION_ENABLED
	NFC_CE_configure(g_sCESupportedModes);

	T4T_CE_init();
#endif


#if NFC_READER_WRITER_ENABLED
	if (g_sRWSupportedModes.ui8byte != 0x00)
	{
		// To be made shortly
		NFC_RW_configure(g_sRWSupportedModes,g_sRWSupportedBitrates);
	}
	ISODEP_configure_RW(g_sRWSetupOptions,g_ui8IsoDepInitiatorDID);
#endif

	// Set the Auto SDD flag within nfc_a.c
	NFC_A_setAutoSDD(g_bEnableAutoSDD);

	// Set the current TRF version within trf79x0.c
	TRF79x0_setVersion(g_eTRFVersion);

	// Set Certification Support for all Protocols - Required for NFC Forum Certification
	NFC_setSupportCertification(g_bSupportCertification);

	// Set Test Enable flag within trf79x0.c - Required for NFC Forum Certification
	TRF79x0_testFlag(g_bSupportCertification);

}

void NFC_initIDs(void)
{
// NFC ID's
	uint8_t pui8NfcAId[10] = {0x08,0x01,0x02,0x03,0x04,0x05,0x06,0x07,0x08,0x09};	// Generic ISO14443 T4TA Tag
	uint8_t pui8NfcBId[4] = {0x08,0x0A, 0xBE,0xEF};	// Generic ISO14443 T4TB Tag
	uint8_t pui8NfcFId[8] = {0x01,0xFE,0x88,0x77,0x66,0x55,0x44,0x33};	// Type F ID for P2P

// Set the NFC Id's for Type A, Type B, and Type F
	NFC_A_setNfcAId(pui8NfcAId,10);
	NFC_B_setNfcBId(pui8NfcBId,4);
	NFC_F_setNfcId2(pui8NfcFId,8);
}

void NFC_emulatedT4TSniffer(void)
{
	uint16_t ui16FileID;
	uint16_t ui16StartIdx;
	uint8_t ui8BufferLength;
	uint8_t ui8T4TStatus;

	char pui8ByteBuffer[3];
	char pui8WordBuffer[5];

	ui8T4TStatus = ISO_7816_4_getCEStatus(&ui16FileID, &ui16StartIdx, &ui8BufferLength);

	if(ui8T4TStatus == CE_READ_FLAG)
	{
		Serial_printfLine("Read  File 0x");

		// Print File ID
		convertWordToAscii(ui16FileID,(uint8_t *)pui8WordBuffer);

		Serial_printfLine(pui8WordBuffer);

		Serial_printfLine(" , Idx 0x");

		// Print Index
		convertWordToAscii(ui16StartIdx,(uint8_t *) pui8WordBuffer);

		Serial_printfLine(pui8WordBuffer);

		Serial_printfLine(" , Len 0x");

		// Print Len
		convertByteToAscii(ui8BufferLength,(uint8_t *) pui8ByteBuffer);

		Serial_printfLine(pui8ByteBuffer);

		Serial_printNewLine(1);

	}
	else if(ui8T4TStatus == CE_WRITE_FLAG)
	{
		Serial_printfLine("Write File 0x");

		// Print File ID
		convertWordToAscii(ui16FileID,(uint8_t *)pui8WordBuffer);

		Serial_printfLine(pui8WordBuffer);

		Serial_printfLine(" , Idx 0x");

		// Print Index
		convertWordToAscii(ui16StartIdx,(uint8_t *) pui8WordBuffer);

		Serial_printfLine(pui8WordBuffer);

		Serial_printfLine(" , Len 0x");

		// Print Len
		convertByteToAscii(ui8BufferLength,(uint8_t *) pui8ByteBuffer);

		Serial_printfLine(pui8ByteBuffer);

		Serial_printNewLine(1);
	}
	else
	{

	}
}

BYTE cdcSendDataWaitTilDone( BYTE* dataBuf, WORD size, BYTE intfNum, ULONG ulTimeout )
{
	return 0;
}

void Serial_printf( char *pui8String, tNFCHostCommands eCmdType )
{
	System_printf( pui8String );
	return;
}

void Serial_printfLine( char *MessagePtr )
{
	System_printf( MessagePtr );
}

void Serial_printNewLine( int value )
{
	int i;

	for( i = 0; i < value; i++ )
		System_printf( "\n" );
}

void Serial_printBuffer( char *pui8String, uint8_t ui8Length, tNFCHostCommands eCmdType )
{
	char TextString[ 256 ];

	memcpy( TextString, pui8String, ui8Length );
	TextString[ ui8Length ] = '\0';
	System_printf( TextString );
	System_flush();

	/* Send data to Queue */
/*
	int Index;

	for( Index = 0; Index < ui8Length - 1; Index++ )
	{
		Report( "0x%x ", pui8String[ Index ] );
	}
	Report( "0x%d\n", pui8String[ Index ] );
*/
}

uint8_t convertNibbleToAscii(uint8_t ui8Nibble)
{
	uint8_t	ui8AsciiOut = ui8Nibble;

	if(ui8Nibble > 9)	// If req ASCII A-F then add 7(hex)
	{
		ui8AsciiOut = ui8AsciiOut + 0x07;
	}

	// Add offset to convert to Ascii
	ui8AsciiOut = ui8AsciiOut + 0x30;

	return(ui8AsciiOut);
}

void convertByteToAscii(uint8_t ui8byte, uint8_t pui8Buffer[3])
{
	uint8_t	ui8Temp1 = 0;

	// Convert High Nibble
	ui8Temp1 = (ui8byte >> 4) & 0x0F;			// get high nibble
	pui8Buffer[0] = convertNibbleToAscii(ui8Temp1);		// convert to ASCII

	// Convert Low Nibble
	ui8Temp1 = ui8byte & 0x0F;					// get low nibble
	pui8Buffer[1] = convertNibbleToAscii(ui8Temp1);		// convert to ASCII

	pui8Buffer[2] = 0x00;
}

void convertWordToAscii( uint16_t ui16Word, uint8_t pui8Buffer[5] )
{
	uint8_t	ui8Temp1 = 0;

// Convert High Nibble
	ui8Temp1 = (ui16Word >> 12) & 0x0F;			// get high nibble
	pui8Buffer[0] = convertNibbleToAscii(ui8Temp1);		// convert to ASCII

	ui8Temp1 = (ui16Word >> 8) & 0x0F;
	pui8Buffer[1] = convertNibbleToAscii(ui8Temp1);		// convert to ASCII

	ui8Temp1 = (ui16Word >> 4) & 0x0F;
	pui8Buffer[2] = convertNibbleToAscii(ui8Temp1);		// convert to ASCII

// Convert Low Nibble
	ui8Temp1 = ui16Word & 0x0F;					// get low nibble
	pui8Buffer[3] = convertNibbleToAscii(ui8Temp1);		// convert to ASCII

	pui8Buffer[4] = 0x00;
}

//*****************************************************************************
//
//! updateLcdfcStatus - Update the current P2P mode information.
//!
//! \param bUpdateRssiOnly defines if the function should update the RSSI value
//! only. When it is set to false, the P2P mode, bitrate, and number of bytes
//! received are also updated on the LCD.
//!
//! Update the activated P2P mode, bitrate, and number of bytes received.
//!
//! \return None.
//
//*****************************************************************************
void updateLcdfcStatus(bool bUpdateRssiOnly)
{
	static uint8_t ui8RssiValue = 0x00;
	char pui8Buffer[3];
	tTRF79x0_Status eTrfStatus;
	uint8_t * pcDataBuffer;
	uint8_t ui8UIDLen;
	uint8_t ui8Temp;

	eTrfStatus = TRF79x0_getStatus();

	if(bUpdateRssiOnly == false)
	{
		// Reset RSSI Value when the other parameters are updated
		ui8RssiValue = 0x00;
		if(eTrfStatus.eCurrentMode == TARGET_MODE )
		{
			if(eTrfStatus.sTargetMode.bits.bPassiveTypeA == 1)
			{
				if(eTrfStatus.bNfcModeEnabled == true)
				{
//					LCD_stringDraw(3,9*6,"Target  ",DOGS102x6_DRAW_NORMAL);

					Serial_printfLine("Passive Target NFC-A Activated");
				}
				else
				{
//					LCD_stringDraw(3,9*6,"CE      ",DOGS102x6_DRAW_NORMAL);

					Serial_printfLine("CE T4TA Activated");
				}

//				LCD_stringDraw(4,9*6,"PASS A  ",DOGS102x6_DRAW_NORMAL);
			}
			else if(eTrfStatus.sTargetMode.bits.bPassiveTypeB == 1)
			{
//				LCD_stringDraw(3,9*6,"CE      ",DOGS102x6_DRAW_NORMAL);
//				LCD_stringDraw(4,9*6,"PASS B  ",DOGS102x6_DRAW_NORMAL);

				Serial_printfLine("CE T4TB Activated");
			}
			else if(eTrfStatus.sTargetMode.bits.bPassiveTypeF == 1 && eTrfStatus.bNfcModeEnabled == true)
			{
//				LCD_stringDraw(3,9*6,"Target  ",DOGS102x6_DRAW_NORMAL);
//				LCD_stringDraw(4,9*6,"PASS F  ",DOGS102x6_DRAW_NORMAL);

				Serial_printfLine("Passive Target NFC-F Activated");
			}
			else if(eTrfStatus.sTargetMode.bits.bActiveTypeA == 1 && eTrfStatus.bNfcModeEnabled == true)
			{
//				LCD_stringDraw(3,9*6,"Target  ",DOGS102x6_DRAW_NORMAL);
//				LCD_stringDraw(4,9*6,"ACTIVE A",DOGS102x6_DRAW_NORMAL);

				Serial_printfLine("Active Target NFC-A Activated");
			}
			else if(eTrfStatus.sTargetMode.bits.bActiveTypeF == 1 && eTrfStatus.bNfcModeEnabled == true)
			{
//				LCD_stringDraw(3,9*6,"Target  ",DOGS102x6_DRAW_NORMAL);
//				LCD_stringDraw(4,9*6,"ACTIVE F",DOGS102x6_DRAW_NORMAL);

				Serial_printfLine("Active Target NFC-F Activated");
			}

			if(eTrfStatus.sTargetPayloadFrequency.bits.b106kpbs == 1)
			{
				if(eTrfStatus.bNfcModeEnabled == true)
				{
					Serial_printf("106 kbps",P2P_BITR_DATA);
				}
//				LCD_stringDraw(5,9*6,"106 kbps",DOGS102x6_DRAW_NORMAL);
			}
			else if(eTrfStatus.sTargetPayloadFrequency.bits.b212kpbs == 1)
			{
				if(eTrfStatus.bNfcModeEnabled == true)
				{
					Serial_printf("212 kbps",P2P_BITR_DATA);
				}
//				LCD_stringDraw(5,9*6,"212 kbps",DOGS102x6_DRAW_NORMAL);
			}
			else if(eTrfStatus.sTargetPayloadFrequency.bits.b424kpbs == 1)
			{
				if(eTrfStatus.bNfcModeEnabled == true)
				{
					Serial_printf("424 kbps",P2P_BITR_DATA);
				}
//				LCD_stringDraw(5,9*6,"424 kbps",DOGS102x6_DRAW_NORMAL);
			}

			Serial_printNewLine(1);
		}
		else if(eTrfStatus.eCurrentMode  == INITIATOR_MODE)
		{
			if(eTrfStatus.sInitiatorMode.bits.bPassiveTypeA == 1)
			{
				Serial_printfLine("NFC-A Tag Detected");

				Serial_printNewLine( 1 );

				Serial_printfLine("ID: ");

				// Get NFC A ID Length
				NFC_A_getNfcAId(&pcDataBuffer,&ui8UIDLen);

				for(ui8Temp=0;ui8Temp<ui8UIDLen;ui8Temp++)
				{
					// Convert hex values to ascii
					convertByteToAscii(pcDataBuffer[ui8Temp],(uint8_t *) pui8Buffer);

					Serial_printfLine(pui8Buffer);
				}

				Serial_printNewLine(1);

				Serial_printfLine("SAK: ");

				// Read SAK Value
				ui8Temp = NFC_A_getSAK();

				// Convert hex values to ascii
				convertByteToAscii(ui8Temp,(uint8_t *) pui8Buffer);

				Serial_printfLine(pui8Buffer);

				Serial_printNewLine(1);

				if(eTrfStatus.bNfcModeEnabled == true)
				{
//					LCD_stringDraw(3,9*6,"Init.   ",DOGS102x6_DRAW_NORMAL);
//					LCD_stringDraw(4,9*6,"PASS A  ",DOGS102x6_DRAW_NORMAL);

					Serial_printfLine("NFC-DEP Enabled ");
				}
				else
				{
					if(ui8Temp & 0x20)
					{
						Serial_printfLine("ISO-DEP Layer Enabled ");
					}
					else
					{
						Serial_printfLine("T2T Layer Enabled ");
					}

//					LCD_stringDraw(3,9*6,"R/W     ",DOGS102x6_DRAW_NORMAL);
//					LCD_stringDraw(4,9*6,"PASS A  ",DOGS102x6_DRAW_NORMAL);
				}
			}
			else if(eTrfStatus.sInitiatorMode.bits.bPassiveTypeB == 1)
			{
				Serial_printfLine("NFC-B Tag Detected");

				Serial_printNewLine(1);

				Serial_printfLine("ID: ");

				// Get NFC B ID Length
				NFC_B_getNfcBId(&pcDataBuffer, &ui8UIDLen);

				for(ui8Temp=0;ui8Temp<ui8UIDLen;ui8Temp++)
				{
					// Convert hex values to ascii
					convertByteToAscii(pcDataBuffer[ui8Temp],(uint8_t *) pui8Buffer);

					Serial_printfLine(pui8Buffer);
				}

				Serial_printNewLine(1);

				if(NFC_B_isISOCompliant())
				{
					Serial_printfLine("ISO-DEP Layer Enabled ");
				}
				else
				{
					Serial_printfLine("ISO-DEP Not supported ");
				}

//				LCD_stringDraw(3,9*6,"R/W     ",DOGS102x6_DRAW_NORMAL);
//				LCD_stringDraw(4,9*6,"PASS B  ",DOGS102x6_DRAW_NORMAL);
			}
			else if(eTrfStatus.sInitiatorMode.bits.bPassiveTypeF == 1)
			{
				Serial_printfLine("NFC-F Tag Detected");

				Serial_printNewLine(1);

				Serial_printfLine("ID: ");

				// Get NFC F ID Length
				NFC_F_getNFCID2(&pcDataBuffer, &ui8UIDLen);

				for(ui8Temp=0;ui8Temp<ui8UIDLen;ui8Temp++)
				{
					// Convert hex values to ascii
					convertByteToAscii(pcDataBuffer[ui8Temp],(uint8_t *) pui8Buffer);

					Serial_printfLine(pui8Buffer);
				}

				if(eTrfStatus.bNfcModeEnabled == true)
				{
//					LCD_stringDraw(3,9*6,"Init.   ",DOGS102x6_DRAW_NORMAL);
//					LCD_stringDraw(4,9*6,"PASS F  ",DOGS102x6_DRAW_NORMAL);

					Serial_printfLine("Passive Initiator NFC-F Activated");
				}
				else
				{
//					LCD_stringDraw(3,9*6,"R/W     ",DOGS102x6_DRAW_NORMAL);
//					LCD_stringDraw(4,9*6,"PASS F  ",DOGS102x6_DRAW_NORMAL);

					Serial_printNewLine(1);

					Serial_printfLine("T3T Layer Enabled ");
				}
			}
			else if(eTrfStatus.sInitiatorMode.bits.bActiveTypeA == 1 && eTrfStatus.bNfcModeEnabled == true)
			{
//				LCD_stringDraw(3,9*6,"Init.   ",DOGS102x6_DRAW_NORMAL);
//				LCD_stringDraw(4,9*6,"ACTIVE A",DOGS102x6_DRAW_NORMAL);

				Serial_printNewLine(1);

				Serial_printfLine("Active Initiator NFC-A Activated");
			}
			else if(eTrfStatus.sInitiatorMode.bits.bActiveTypeF == 1 && eTrfStatus.bNfcModeEnabled == true)
			{
//				LCD_stringDraw(3,9*6,"Init.   ",DOGS102x6_DRAW_NORMAL);
//				LCD_stringDraw(4,9*6,"ACTIVE F",DOGS102x6_DRAW_NORMAL);

				Serial_printNewLine(1);

				Serial_printfLine("Active Initiator NFC-F Activated");
			}
			else if(eTrfStatus.sInitiatorMode.bits.bPassive15693 == 1)
			{
				Serial_printfLine("NFC-V Tag Detected");

				Serial_printNewLine(1);

				Serial_printfLine("UID: ");

				// Get NFC V ID Length
				NFC_RW_T5T_getT5TUID(&pcDataBuffer,&ui8UIDLen);

				for(ui8Temp=ui8UIDLen;ui8Temp>0;ui8Temp--)
				{
					// Convert hex values to ascii
					convertByteToAscii(pcDataBuffer[ui8Temp-1],(uint8_t *) pui8Buffer);

					Serial_printfLine(pui8Buffer);
				}

//				LCD_stringDraw(3,9*6,"R/W     ",DOGS102x6_DRAW_NORMAL);
//				LCD_stringDraw(4,9*6,"15693   ",DOGS102x6_DRAW_NORMAL);
			}

			if(eTrfStatus.sInitiatorPayloadFrequency.bits.b26_48_kbps_1_out_4 == 1)
			{
//				LCD_stringDraw(5,9*6,"26.5kbps",DOGS102x6_DRAW_NORMAL);
			}
			else if(eTrfStatus.sInitiatorPayloadFrequency.bits.b106kpbs == 1)
			{
				if(eTrfStatus.bNfcModeEnabled == true)
				{
					Serial_printf("106 kbps",P2P_BITR_DATA);
				}
//				LCD_stringDraw(5,9*6,"106 kbps",DOGS102x6_DRAW_NORMAL);
			}
			else if(eTrfStatus.sInitiatorPayloadFrequency.bits.b212kpbs == 1)
			{
				if(eTrfStatus.bNfcModeEnabled == true)
				{
					Serial_printf("212 kbps",P2P_BITR_DATA);
				}
//				LCD_stringDraw(5,9*6,"212 kbps",DOGS102x6_DRAW_NORMAL);
			}
			else if(eTrfStatus.sInitiatorPayloadFrequency.bits.b424kpbs == 1)
			{
				if(eTrfStatus.bNfcModeEnabled == true)
				{
					Serial_printf("424 kbps",P2P_BITR_DATA);
				}
//				LCD_stringDraw(5,9*6,"424 kbps",DOGS102x6_DRAW_NORMAL);
			}
			else if(eTrfStatus.sInitiatorPayloadFrequency.bits.b848kpbs == 1)
			{
//				LCD_stringDraw(5,9*6,"848 kbps",DOGS102x6_DRAW_NORMAL);
			}

			Serial_printNewLine(1);
		}

//		LCD_stringDraw(6,9*6,"0x",DOGS102x6_DRAW_NORMAL);
//		LCD_stringDraw(7,9*6,"0x",DOGS102x6_DRAW_NORMAL);

		ui8RssiValue = eTrfStatus.ui8InternalRssi;
		convertByteToAscii(eTrfStatus.ui8InternalRssi,(uint8_t *)pui8Buffer);
//		LCD_stringDraw(7,11*6,pui8Buffer,DOGS102x6_DRAW_NORMAL);
		Serial_printf(pui8Buffer,P2P_RSSI_DATA);
	}

	if(eTrfStatus.ui8InternalRssi != ui8RssiValue)
	{
		ui8RssiValue = eTrfStatus.ui8InternalRssi;
		convertByteToAscii(eTrfStatus.ui8InternalRssi,(uint8_t *)pui8Buffer);
//		LCD_stringDraw(7,11*6,pui8Buffer,DOGS102x6_DRAW_NORMAL);
		Serial_printf(pui8Buffer,P2P_RSSI_DATA);
		Serial_printNewLine(1);
	}
}

// *****************************************************************************
// end of file
// *****************************************************************************

